#include "Battle.h"
#include "DrawWindow.h"
#include "Combatant.h"

#include "SDL.h"
#include "SDL_gfxPrimitives.h"
#include "SDL_ttf.h"

#include "SDL_image.h"
#include "Enemy.h"

#include <iostream>
#include <ostream>
#include <sstream>
using namespace std;

#include "InputObj.h"
#include "UserInput.h"
#include <stdlib.h>
#include "sdMath.h"

#define BattleStateWaitingForInit 0
#define BattleStateBattleBegun 1
#define BattleStateChooseAttack 2
#define BattleStatePlayerAttackBegin 3
#define BattleStateEnemyAttackBegin 4
#define BattleStateRoundOver 5
#define BattleStateBattleOver 6
#define BattleStateYouLose 7

Battle :: Battle ()
{
  battleState = BattleStateWaitingForInit;
  lastTicks = SDL_GetTicks ();
  numVicts = 0;
  countVict = false;
  countLoss = false;
  leftCombatant = NULL;
  rightCombatant = NULL;
  currentCombatant = NULL;
  lastTicks = SDL_GetTicks ();
  tmVec.clear ();
  tmVec.resize (0);
  font = NULL;
  ostringstream oss (ostringstream::out);
  roundCount = 0;
  roundsSpentDuringThisFight = 0;
  numAttacks = 0;

  dw = new DrawWindow ();

  //load backgrounds here
  string tempFilename ("media/bkground/woodCrap.png");
  bkg = IMG_Load(tempFilename.c_str());

  if (bkg == NULL)
    {cout << "Image Loading error:" << IMG_GetError ();cout.flush ();}

  enem = NULL;
  enem = new Enemy ();
}

void Battle :: reset ()
{
  //clears all relevant data, ready to restart game.
  battleState = BattleStateWaitingForInit;
  lastTicks = SDL_GetTicks ();
  numVicts = 0;
  countVict = false;
  countLoss = false;
  leftCombatant = NULL;
  rightCombatant = NULL;
  currentCombatant = NULL;
  roundCount = 0;
  roundsSpentDuringThisFight = 0;
  totalDmg = 0;
  numHits = 0;
  numAttacks = 0;

  delete dw;
  dw = new DrawWindow ();

  enem -> setUpBadguy (numVicts);

  stats.clear ();
}

void Battle :: init (Combatant * lComb, TTF_Font * f)
{
  //gets the involved parties, loads stats.
  //update parameters. main calls this at the beginning of a battle.

  Uint32 numanims, animToPick;

  endurance = 1000;
  battleState = BattleStateBattleBegun;
  lastTicks = SDL_GetTicks ();

  overkillMode = 0;

  leftCombatant = lComb;
  if (numVicts % 5 == 0)
    {
      numanims = leftCombatant -> numberOfPossibleAnims (4);

      if (numanims > 0)
	animToPick = randomUniformInt (numanims);
      else
	animToPick = 0;

      leftCombatant -> animate (4, animToPick);
    }
  else
    leftCombatant -> animate (0, 0);

  enem -> setUpBadguy (numVicts);
  rightCombatant = enem -> getCombatant ();
  addStats1 ();

  numanims = rightCombatant -> numberOfPossibleAnims (4);
  if (numanims > 0)
    animToPick = randomUniformInt (numanims);
  else
    animToPick = 0;
  
  rightCombatant -> animate (4, animToPick);

  currentCombatant = leftCombatant;

  numHits = 0;

  Uint32 numPatterns = 1, numExtraPoints = 1;
  Uint32 chanceOfDoublePatterns;

  numExtraPoints = 16 + (16 * numVicts / 25);

  chanceOfDoublePatterns = numVicts * numVicts / 9;
  if (chanceOfDoublePatterns > 64)
    chanceOfDoublePatterns = 64;

  if (randomUniformInt (256) < chanceOfDoublePatterns)
    {numPatterns = 2; enem -> halvePatternSize ();
    numExtraPoints = 16 + (16 * numVicts / 50);
    }

  dw -> init (leftCombatant, f, enem, numPatterns, numExtraPoints);

  font = f;
}

void Battle :: render (SDL_Surface * dest)
{
  string s;

  SDL_Color textCol;
  textCol.r = 255;
  textCol.g = 255;
  textCol.b = 255;

  SDL_Color textCol2;
  textCol2.r = 64;
  textCol2.g = 64;
  textCol2.b = 64;

  SDL_Rect textPosa;
  textPosa.x = 0;
  textPosa.y = 0;

  SDL_Rect textPos1;
  textPos1.x = 170;
  textPos1.y = 350;

  SDL_Rect textPos2;
  textPos2.x = 170;
  textPos2.y = 369;

  //paste background here
  
  if (bkg == NULL)
    SDL_FillRect(dest, NULL, SDL_MapRGB(dest -> format, 0, 0, 0));
  else
    SDL_BlitSurface(bkg, NULL, dest, NULL);

  s.append ("Escape = Quit, f = fullscreen, F10 = restart");
  textOutput = TTF_RenderText_Solid (font, s.c_str (), textCol);
  SDL_BlitSurface(textOutput, NULL, dest, &textPosa);
  SDL_FillRect(textOutput, NULL, SDL_MapRGB(dest -> format, 0, 0, 0));
  SDL_FreeSurface (textOutput);

  //render based on the BattleState

  renderEndurance (dest, textCol, textCol2, textPos1);
  if ((battleState == BattleStatePlayerAttackBegin) || (battleState == BattleStateRoundOver))
    {
      if (currentCombatant == leftCombatant)
	{
	  dw -> render (dest, overkillMode);
	  if (leftCombatant -> getIo () != NULL)
	    leftCombatant -> getIo () -> render (dest);
	}
    }

  if (battleState == BattleStateBattleBegun)
    renderBattleIntro (dest, textCol, textPos1);

  if (battleState == BattleStateBattleOver)
      renderBattleExtro (dest, textCol, textPos1);

  renderCombatants (dest);
  renderStats (dest, textCol);
  renderTextMessages (dest);
}

void Battle :: addMessage (string & s, SDL_Color fg, SDL_Color * bg, Uint32 dur, SDL_Rect & pos)
{
  //adds a textMessage.
  textMessage * tm = NULL;

  //make it
  tm = new textMessage (s, fg, bg, dur, pos, font);

  tmVec.push_back (tm);
}

void Battle :: addMessage (textMessage * tm)
{
  tmVec.push_back (tm);
}

void Battle :: renderTextMessages (SDL_Surface * dest)
{
  //removes expired textMessages and displays the rest.
  vector <textMessage *> :: iterator iter;

  iter = tmVec.begin ();

  while (iter < tmVec.end ())
    {
      //if it's expired, remove it.
      if ((*iter) -> done)
	{
	  //remove it.
	  delete (*iter);
	  tmVec.erase(iter);
	}
      else
	iter ++;
    }

  iter = tmVec.begin ();

  while (iter < tmVec.end ())
    {
      (*iter) -> render (dest);
      iter ++;
    }
}

void Battle :: renderCombatants (SDL_Surface * dest)
{
  //players should draw
  leftCombatant -> render (dest, font); 
  rightCombatant -> render (dest, font); 

  SDL_Color textCol;
  textCol.r = 176;
  textCol.g = 176;
  textCol.b = 176;

  SDL_Rect tPos;
  tPos.x = 320;
  tPos.y = 380;

  oss.str("");
  oss << "Wins: " << Uint32(numVicts);
  textOutput = TTF_RenderText_Solid (font, oss.str ().c_str (), textCol);
  tPos.x -= textOutput -> w / 2;
  SDL_BlitSurface(textOutput, NULL, dest, &tPos);
  SDL_FreeSurface (textOutput);

  tPos.x = 260;
  tPos.y = 410;
  oss.str("");
  oss << "Damage: " << Uint32(totalDmg);
  textOutput = TTF_RenderText_Solid (font, oss.str ().c_str (), textCol);
  tPos.x -= textOutput -> w / 2;
  SDL_BlitSurface(textOutput, NULL, dest, &tPos);
  SDL_FreeSurface (textOutput);

  tPos.x = 380;
  tPos.y = 410;
  oss.str("");
  oss << "Hits: " << Uint32(numAttacks);
  textOutput = TTF_RenderText_Solid (font, oss.str ().c_str (), textCol);
  tPos.x -= textOutput -> w / 2;
  SDL_BlitSurface(textOutput, NULL, dest, &tPos);
  SDL_FreeSurface (textOutput);
}

void Battle :: renderBattleIntro (SDL_Surface * dest, SDL_Color & textCol, SDL_Rect & textPos1)
{
  string s;
  if (rightCombatant != NULL)
    {
      s.assign (rightCombatant -> getName ());
    }

  s.append (" approaches!");
  textOutput = TTF_RenderText_Solid (font, s.c_str(), textCol);
  SDL_BlitSurface(textOutput, NULL, dest, &textPos1);
  SDL_FreeSurface (textOutput);
}

void Battle :: renderBattleExtro (SDL_Surface * dest, SDL_Color & textCol, SDL_Rect & textPos1)
{
  oss.str ("");

  if (leftCombatant -> getAreYouDead ())
    oss << "OH NO..." << numVicts << " victories total.";
  else
    oss << "OH YEAH!! ";

  textOutput = TTF_RenderText_Solid (font, oss.str ().c_str (), textCol);
  textPos1.x = 320 - (textOutput -> w / 2);
  SDL_BlitSurface(textOutput, NULL, dest, &textPos1);
  textPos1.x = 320 + (textOutput -> w / 2);
  SDL_FreeSurface (textOutput);
}

void Battle :: renderEndurance (SDL_Surface * dest, SDL_Color & textCol, SDL_Color & textCol2, SDL_Rect & textPos1)
{
  if (!(battleState == BattleStatePlayerAttackBegin))
    return;

  string s;

  Sint16 endFillx = 320, endFillw = 100, endFillwMax = 100, endFillh = 25, endFillhMax = 25;
  Sint16 endFilly = 330 + endFillhMax + 5;

  Uint8 endFillr, endFillg, endFillb;

  switch (overkillMode)
    {
    case (0): endFillr = 231; endFillg = 169; endFillb = 0; break;
    case (1): endFillr = 191; endFillg = 150; endFillb = 38; break;
    case (2): default: endFillr = 0; endFillg = 64; endFillb = 0; break;
    }

  //draw based on battle state.
  if (dw -> getDrawingStatus () == 1)
    {
      s.append ("Connect the dots with the mouse.");
      textOutput = TTF_RenderText_Solid (font, s.c_str (), textCol);
      SDL_BlitSurface(textOutput, NULL, dest, &textPos1);
      SDL_FreeSurface (textOutput);
    }
  if (dw -> getDrawingStatus () == 2 || dw -> getDrawingStatus () == 3 || dw -> getDrawingStatus () == 5)
    {

      filledEllipseRGBA(dest, endFillx, endFilly, endFillwMax, endFillhMax, 64, 64, 64, 128);

      endFillh = endFillhMax * getEndurance() / 1000;
      endFillw = endFillwMax * getEndurance() / 1000;

      if (getEndurance() < 0 || getEndurance() > 1000)
	{endFillh = endFillhMax; endFillw = endFillwMax;}

      filledEllipseRGBA(dest, endFillx, endFilly, endFillw, endFillh, endFillr, endFillg, endFillb, 255);

      //aa
      ellipseRGBA(dest, endFillx, endFilly, endFillwMax, endFillhMax, 64, 64, 64, 255);
      filledEllipseRGBA(dest, endFillx, endFilly, endFillw, endFillh, endFillr, endFillg, endFillb, 255);
    }
}

void Battle :: renderStats (SDL_Surface * dest, SDL_Color & textCol)
{
  Sint16 pax = 80, pay = 400, pas = 15, pasm = 25, par = 128, pab = 0, pag = 0, paa = 255;
  Sint16 pdxv [5];
  Sint16 pdyv [5];
  Sint16 pdx = 140, pdy = 400, pds = 15, pdsm = 25, pdr = 0, pdb = 0, pdg = 0, pda = 255;

  Sint16 phx = 120, phy = 400, phs = 30, phsm = 50, phr = 0, phb = 0, phg = 64, pha = 255;

  SDL_Rect textPos;
  Uint32 lvl, next;
  Sint32 curr;

  switch (roundCount % 5)
    {
    case (0): case (1): case (2): 
      pdr = 107; pdg = 21; pdb = 84;
    case (3): pdr = 70; pdg = 55; pdb = 14; break;
    default: pdr = 0; pdb = 0; pdg = 0; break;
    }
  
  switch (roundCount % 5)
    {
    case (0): case (1): case (2): 
      par = 107; pab = 21; pag = 84;
    case (3): par = 70; pab = 55; pag = 14; break;
    default: par = 0; pab = 0; pag = 0; break;
    }

  //draw stats: Vary the brightness
  if (overkillMode != 2) //draw attack & defense if in normal mode
    {
      leftCombatant -> getAtkStats (&lvl, &curr, &next);
      pas = pasm * (curr + 1) / next;
      //paa = 255 * (curr + 1) / next;

      filledTrigonRGBA(dest, pax, (-80 * pasm / 100) + pay, 
		       (-40 * pasm / 100) + pax, (80 * pasm / 100) + pay, 
		       (40 * pasm / 100) + pax, (80 * pasm / 100) + pay,
		       64, 64, 64, 128);
      filledTrigonRGBA(dest, pax, (-80 * pas / 100) + pay, 
		       (-40 * pas / 100) + pax, (80 * pas / 100) + pay, 
		       (40 * pas / 100) + pax, (80 * pas / 100) + pay,
		       par, pab, pag, paa);
      //aa
      trigonRGBA(dest, pax, (-80 * pasm / 100) + pay, 
		   (-40 * pasm / 100) + pax, (80 * pasm / 100) + pay, 
		   (40 * pasm / 100) + pax, (80 * pasm / 100) + pay,
		   64, 64, 64, 255);
      
      //insert level here.
      oss.str ("");
      oss << lvl;
      textOutput = TTF_RenderText_Solid (font, oss.str ().c_str (), textCol);
      textPos.x = pax - (textOutput -> w/2);
      textPos.y = pay - (textOutput -> h/2);
      SDL_BlitSurface(textOutput, NULL, dest, &textPos);
      SDL_FreeSurface (textOutput);

      leftCombatant -> getDefStats (&lvl, &curr, &next);
      pds = pdsm * (curr + 1) / next;
      //pda = 255 * (curr + 1) / next;
      
      pdxv[0] = (-80 * pdsm / 100) + pdx;
      pdxv[1] = (80 * pdsm / 100) + pdx;
      pdxv[2] = (60 * pdsm / 100) + pdx;
      pdxv[3] = (0 * pdsm / 100) + pdx;
      pdxv[4] = (-60 * pdsm / 100) + pdx;

      pdyv[0] = (-60 * pdsm / 100) + pdy;
      pdyv[1] = (-60 * pdsm / 100) + pdy;
      pdyv[2] = (50 * pdsm / 100) + pdy;
      pdyv[3] = (80 * pdsm / 100) + pdy;
      pdyv[4] = (50 * pdsm / 100) + pdy;

      filledPolygonRGBA(dest, pdxv, pdyv, 5, 64, 64, 64, 128);
	  
      pdxv[0] = (-80 * pds / 100) + pdx;
      pdxv[1] = (80 * pds / 100) + pdx;
      pdxv[2] = (60 * pds / 100) + pdx;
      pdxv[3] = (0 * pds / 100) + pdx;
      pdxv[4] = (-60 * pds / 100) + pdx;

      pdyv[0] = (-60 * pds / 100) + pdy;
      pdyv[1] = (-60 * pds / 100) + pdy;
      pdyv[2] = (50 * pds / 100) + pdy;
      pdyv[3] = (80 * pds / 100) + pdy;
      pdyv[4] = (50 * pds / 100) + pdy;

      filledPolygonRGBA(dest, pdxv, pdyv, 5, pdr, pdg, pdb, pda);
      
      pdxv[0] = (-80 * pdsm / 100) + pdx;
      pdxv[1] = (80 * pdsm / 100) + pdx;
      pdxv[2] = (60 * pdsm / 100) + pdx;
      pdxv[3] = (0 * pdsm / 100) + pdx;
      pdxv[4] = (-60 * pdsm / 100) + pdx;

      pdyv[0] = (-60 * pdsm / 100) + pdy;
      pdyv[1] = (-60 * pdsm / 100) + pdy;
      pdyv[2] = (50 * pdsm / 100) + pdy;
      pdyv[3] = (80 * pdsm / 100) + pdy;
      pdyv[4] = (50 * pdsm / 100) + pdy;
      //aa
      polygonRGBA(dest, pdxv, pdyv, 5, 64, 64, 64, 255);
      
      //insert level here.
      oss.str ("");
      oss << lvl;
      textOutput = TTF_RenderText_Solid (font, oss.str ().c_str (), textCol);
      textPos.x = pdx - (textOutput -> w/2);
      textPos.y = pdy - (textOutput -> h/2);
      SDL_BlitSurface(textOutput, NULL, dest, &textPos);
      SDL_FreeSurface (textOutput);
    }
  else //draw healing if in overkill mode.
    {
      leftCombatant -> getHealStats (&lvl, &curr, &next);
      phs = phsm * (curr + 1) / next;
      //pha = 255 * (curr + 1) / next;
      
      boxRGBA(dest, phx - (10 * phsm / 100), phy - (50 * phsm / 100), phx + (10 * phsm / 100), phy + (50 * phsm / 100), 64, 64, 64, 128);
      boxRGBA(dest, phx - (50 * phsm / 100), phy - (10 * phsm / 100), phx + (50 * phsm / 100), phy + (10 * phsm / 100), 64, 64, 64, 128);
      
      boxRGBA(dest, phx - (10 * phs / 100), phy - (50 * phs / 100), phx + (10 * phs / 100), phy + (50 * phs / 100), phr, phg, phb, pha);
      boxRGBA(dest, phx - (50 * phs / 100), phy - (10 * phs / 100), phx + (50 * phs / 100), phy + (10 * phs / 100), phr, phg, phb, pha);

      //insert level here.
      oss.str ("");
      oss << lvl;
      textOutput = TTF_RenderText_Solid (font, oss.str ().c_str (), textCol);
      textPos.x = phx - (textOutput -> w/2);
      textPos.y = phy - (textOutput -> h/2);
      SDL_BlitSurface(textOutput, NULL, dest, &textPos);
      SDL_FreeSurface (textOutput);
    }

  //now for the enemy's stats:
  pax = 500; pdx = 560; paa = 192; pda = 192;

  rightCombatant -> getAtkStats (&lvl, &curr, &next);
  filledTrigonRGBA(dest, pax, (-80 * pasm / 100) + pay, 
		   (-40 * pasm / 100) + pax, (80 * pasm / 100) + pay, 
		   (40 * pasm / 100) + pax, (80 * pasm / 100) + pay,
		   par, pab, pag, paa);
  oss.str ("");
  oss << lvl;
  textOutput = TTF_RenderText_Solid (font, oss.str ().c_str (), textCol);
  textPos.x = pax - (textOutput -> w/2);
  textPos.y = pay - (textOutput -> h/2);
  SDL_BlitSurface(textOutput, NULL, dest, &textPos);
  SDL_FreeSurface (textOutput);

  rightCombatant -> getDefStats (&lvl, &curr, &next);
  pdxv[0] = (-80 * pdsm / 100) + pdx;
  pdxv[1] = (80 * pdsm / 100) + pdx;
  pdxv[2] = (60 * pdsm / 100) + pdx;
  pdxv[3] = (0 * pdsm / 100) + pdx;
  pdxv[4] = (-60 * pdsm / 100) + pdx;

  pdyv[0] = (-60 * pdsm / 100) + pdy;
  pdyv[1] = (-60 * pdsm / 100) + pdy;
  pdyv[2] = (50 * pdsm / 100) + pdy;
  pdyv[3] = (80 * pdsm / 100) + pdy;
  pdyv[4] = (50 * pdsm / 100) + pdy;

  filledPolygonRGBA(dest, pdxv, pdyv, 5, pdr, pdg, pdb, pda);
  oss.str ("");
  oss << lvl;
  textOutput = TTF_RenderText_Solid (font, oss.str ().c_str (), textCol);
  textPos.x = pdx - (textOutput -> w/2);
  textPos.y = pdy - (textOutput -> h/2);
  SDL_BlitSurface(textOutput, NULL, dest, &textPos);
  SDL_FreeSurface (textOutput);
}

void Battle :: animateAttack (Combatant * attacker, Combatant * defender, Uint32 damage)
{
  Uint32 attackerSubAnim = 0;
  Uint32 defenderSubAnim = 0;
  Combatant * partnerAnim = defender;
  Sint32 attDiff = attacker -> getAtkLevel () - defender -> getDefLevel ();
  Sint32 percentDmg = damage * 100;

  if ((defender -> getHp () + damage) == 0)
    percentDmg = 100;
  else
    percentDmg /= (defender -> getHp () + damage);

  if (defender -> getAreYouDead ())
    partnerAnim = NULL;

  //attacker: +2 levels or over 1/3 hp damage is strong.
  if (percentDmg >= 33)
    attackerSubAnim = 1;
  else if (attDiff >= 2)
    attackerSubAnim = 1;

  //attacker: -2 levels or less than 1/10 hp damage is weak.
  if (percentDmg <= 10)
    attackerSubAnim = 2;
  else if (attDiff <= -2)
    attackerSubAnim = 2;

  //defender: You just care about how much damage you took.
  if (percentDmg >= 33)
    defenderSubAnim = 1;
  
  if (percentDmg <= 10)
    defenderSubAnim = 2;

  attacker -> animate (1, attackerSubAnim, partnerAnim, 2, defenderSubAnim);
}

void Battle :: renderResults (SDL_Surface * dest, SDL_Color & textCol, SDL_Rect & textPos1)
{
}

void Battle :: gameLoop ()
{
  SDL_Event enemyMade;
  Uint32 numanims, animToPick;

  //battleState
  //0: Waiting for initialization
  //1: Announce that battle has begun!
  //2: In battle. Draw, attack, deal damage. (May have to expand this.)
  //3: Round over. Show results of the round. Go back to step 2.
  //4: The fight is over, show results.
  switch (battleState){
  case BattleStateWaitingForInit:
    break;
  case BattleStateBattleBegun:
    if (introIsOver ())
      {startOfFight ();}
    break;
  case BattleStateChooseAttack:
    leftCombatant -> animate (0, 0); rightCombatant -> animate (0, 0);

    if (currentCombatant == leftCombatant)
      {
	battleState = BattleStatePlayerAttackBegin;
	startOfRound ();
      }
    else
      battleState = BattleStateEnemyAttackBegin;
    break;
  case BattleStatePlayerAttackBegin:
    if (leftCombatant -> getIo () != NULL){
      dw -> drawEvaluator (leftCombatant -> getIo (), endurance, numHits, overkillMode, rightCombatant -> getDefLevel (), leftCombatant -> getAtkLevel ());
    }

    if (dw -> getDrawingStatus () == 4)
      {
	endOfRound ();
      }
    break;
  case BattleStateEnemyAttackBegin:
    //if it's the AI's turn, tell it to do it's AI thing.
    enemyStruct = new SDL_Rect ();
    enemyStruct -> w = 1;
    enemyStruct -> h = 8; //how much damage is inflicted?
    
    enemyMade.type = SDL_USEREVENT;
    enemyMade.user.code = 0;
    enemyMade.user.data1 = (void *) enemyStruct;
    enemyMade.user.data2 = 0;
    SDL_PushEvent(&enemyMade);

    endOfRound ();
    break;
  case BattleStateRoundOver:
    if (extroIsOver ())
      {
	lastTicks = SDL_GetTicks ();
	degradeStats ();

	//check to see if the battle is over.
	if (rightCombatant -> getAreYouDead () || leftCombatant -> getAreYouDead ())
	  {
	    battleState = BattleStateBattleOver;
	  }
	else
	  {
	    //otherwise, Switch turns, the round is over.
	    switchTurns ();
	  }
      }
    break;
  case BattleStateBattleOver:
    if (rightCombatant -> getAreYouDead ())
      addVictory ();

    if (rightCombatant -> getAreYouDead () && (SDL_GetTicks () - lastTicks > 2000))
      endOfFight ();
    else if (leftCombatant -> getAreYouDead ())
      {
	if (SDL_GetTicks () - lastTicks > 1000 && (countLoss == false))
	  {
	    numanims = rightCombatant -> numberOfPossibleAnims (5);
	    
	    if (numanims > 0)
	      animToPick = randomUniformInt (numanims);
	    else
	      animToPick = 0;
	    
	    rightCombatant -> animate (5, animToPick);
	    countLoss = true;
	  }
	if (SDL_GetTicks () - lastTicks > 5000)
	  {
	    lastTicks = SDL_GetTicks ();
	    battleState = BattleStateYouLose;
	    addStats2 ();
	  }
      }
    break;
  default:
    break;
  }
}

void Battle :: switchTurns ()
{
  if (currentCombatant == leftCombatant)
    currentCombatant = rightCombatant;
  else
    currentCombatant = leftCombatant;
  
  endurance = 1000;
  battleState = BattleStateChooseAttack;
  leftCombatant -> getIo () -> clearInput ();
  dw -> clear ();
}

bool Battle :: introIsOver ()
{
  if (SDL_GetTicks () - lastTicks > 1000)
    return true;

  return false;
}

bool Battle :: extroIsOver ()
{
  if (SDL_GetTicks () - lastTicks > 1000)
    return true;

  return false;
}

void Battle :: degradeStats ()
{
  string sssss;
  textMessage * tm = NULL;

  if (currentCombatant == leftCombatant)
    {
      numHits = 0;
      roundCount ++;
      roundsSpentDuringThisFight ++;

      if (roundCount % 5 == 0)
	{
	  leftCombatant -> loseAtkLevel (1);
	  sssss.assign ("Atk Down");
	  tm = new textMessage (sssss, 1, font, 20, 300, 5000);
	  addMessage (tm);

	  leftCombatant -> loseDefLevel (1);
	  sssss.assign ("Def Down");
	  tm = new textMessage (sssss, 1, font, 85, 300, 5000);
	  addMessage (tm);
	}
    }
}

void Battle :: processInput (const SDL_Event & mainEvent)
{
  //give control to whichever player is in action
  Uint16 targetC;
  Uint16 boostAmnt, boostIndex;
  Sint32 levelUp = 0;
  Uint32 numDmg, nHits;
  Uint32 attlvl;

  string sssss;
  textMessage * tm = NULL;

  switch(mainEvent.type){
  case SDL_USEREVENT:
    //read the struct
    switch (mainEvent.user.code)
      {
      case (0): //attack hit
	targetC = ((SDL_Rect *) (mainEvent.user.data1)) -> w;

	//calculate damage here
	numDmg = ((SDL_Rect *) (mainEvent.user.data1)) -> h;
	nHits = ((SDL_Rect *) (mainEvent.user.data1)) -> x;

	oss.str ("");
	
	//deal damage to target
	if (targetC != 1)
	  {
	    leftCombatant -> getAtkStats (&attlvl, NULL, NULL);

	    //overkill bonus
	    if (rightCombatant -> takeDamage (numDmg, nHits, attlvl))
	      {
		overkillMode = 2; 
		if (endurance < 250)
		  endurance = 250;

		if (dw -> getEnduranceLossRate () > 250)
		  dw -> setEnduranceLossRate (250);
	      }

	    numAttacks ++;
	    totalDmg += numDmg;

	    oss << numDmg;
	    sssss.assign (oss.str ());
	    tm = new textMessage (sssss, 0, font, leftCombatant -> getIo () -> getCursorPosX (), leftCombatant -> getIo () -> getCursorPosY ());
	  }
	else
	  {
	    rightCombatant -> getAtkStats (&attlvl, NULL, NULL);
	    leftCombatant -> takeDamage (numDmg, 1, attlvl);

	    oss << numDmg;
	    sssss.assign (oss.str ());
	    tm = new textMessage (sssss, 0, font, 85, 330, 2000);
	  }

	if (targetC == 0)
	  animateAttack (leftCombatant, rightCombatant, numDmg);
	else
	  animateAttack (rightCombatant, leftCombatant, numDmg);

	addMessage (tm);
	break;
      case (1): //stat boost
	boostIndex = ((Uint32) (mainEvent.user.data1));
	boostAmnt = ((Uint32) (mainEvent.user.data2));

	sssss = oss.str ();	

	switch (boostIndex)
	  {
	  case (1): 
	    levelUp = leftCombatant -> addAtkPoints (boostAmnt);

	    if (levelUp > 0)
	      {sssss.assign ("Atk Up!"); 
	      tm = new textMessage (sssss, 1, font, leftCombatant -> getIo () -> getCursorPosX (), leftCombatant -> getIo () -> getCursorPosY ());
	      addMessage (tm);}
	    else if (levelUp < 0)
	      {sssss.assign ("Atk down"); 
	      tm = new textMessage (sssss, 1, font, leftCombatant -> getIo () -> getCursorPosX (), leftCombatant -> getIo () -> getCursorPosY ());
	      addMessage (tm);}

	    break;
	  case (2):
	    levelUp = leftCombatant -> addDefPoints (boostAmnt); 

	    if (levelUp > 0)
	      {sssss.assign ("Def Up!"); 
	      tm = new textMessage (sssss, 1, font, leftCombatant -> getIo () -> getCursorPosX (), leftCombatant -> getIo () -> getCursorPosY ());
	      addMessage (tm);}
	    else if (levelUp < 0)
	      {sssss.assign ("Def down"); 
	      tm = new textMessage (sssss, 1, font, leftCombatant -> getIo () -> getCursorPosX (), leftCombatant -> getIo () -> getCursorPosY ());
	      addMessage (tm);}
	    break;
	  case (3): 
	    levelUp = leftCombatant -> addHealPoints (boostAmnt); 

	    if (levelUp > 0)
	      {sssss.assign ("HP!"); 
	      tm = new textMessage (sssss, 2, font, leftCombatant -> getIo () -> getCursorPosX (), leftCombatant -> getIo () -> getCursorPosY ());
	      addMessage (tm);}
	    break;
	  default: break;
	  }
	break;
      case (2): //text message should be drawn
	sssss.assign (* (string *) (mainEvent.user.data1));
	targetC = ((Uint32) (mainEvent.user.data2));

	tm = new textMessage (sssss, targetC, font, leftCombatant -> getIo () -> getCursorPosX (), leftCombatant -> getIo () -> getCursorPosY ());
	addMessage (tm);

	break;
      default: break;
      }
    break;
  default:
    break;
  }

  if (leftCombatant != NULL && leftCombatant -> getIo () != NULL)
    leftCombatant -> getIo () -> processInput (mainEvent);
}

bool Battle :: ifPlayerAttIsGreaterThanEnemyDef ()
{
  Uint32 enemDeflvl, playerAtklvl;
  
  rightCombatant -> getDefStats (&enemDeflvl, NULL, NULL);
  leftCombatant -> getAtkStats (&playerAtklvl, NULL, NULL);

  if (playerAtklvl > enemDeflvl)
    return true;
  else
    return false;
}

void Battle :: addVictory ()
{
  Uint32 numanims, animToPick;

  //adds a victory...
  if (countVict == true)
    return;

  numVicts ++;
  addStats2 ();
  leftCombatant -> emptyHealLevels ();

  numanims = leftCombatant -> numberOfPossibleAnims (5);

  if (numanims > 0)
    animToPick = randomUniformInt (numanims);
  else
    animToPick = 0;

  leftCombatant -> animate (5, animToPick);

  countVict = true;
  overkillMode = 0;
  return;
}

void Battle :: startOfRound ()
{
  totalDmg = 0; numAttacks = 0;
}

void Battle :: endOfRound ()
{
  battleState = BattleStateRoundOver;
  lastTicks = SDL_GetTicks ();
}

void Battle :: startOfFight ()
{
  currentCombatant = leftCombatant;
  battleState = BattleStateChooseAttack;
  lastTicks = SDL_GetTicks ();
}

void Battle :: endOfFight ()
{
  lastTicks = SDL_GetTicks ();
  countVict = false;
  countLoss = false;
  roundsSpentDuringThisFight = 0;
  battleState = BattleStateWaitingForInit;
  leftCombatant -> animate (0, 0); rightCombatant -> animate (0, 0);
}

void Battle :: addStats1 ()
{
  Sint32 atkComp, defComp;
  atkComp = leftCombatant -> getAtkLevel () - rightCombatant -> getDefLevel ();
  defComp = leftCombatant -> getDefLevel () - rightCombatant -> getAtkLevel ();

  stats.recordStats1 (numVicts, enem -> getEnemyIndex (), 
		     leftCombatant -> getAtkLevel (), 
		     leftCombatant -> getDefLevel (), 
		     atkComp, defComp);
  
}

void Battle :: addStats2 ()
{
  Sint32 atkComp, defComp;
  atkComp = leftCombatant -> getAtkLevel () - rightCombatant -> getDefLevel ();
  defComp = leftCombatant -> getDefLevel () - rightCombatant -> getAtkLevel ();

  stats.recordStats2 (leftCombatant -> getHp (), leftCombatant -> getHealLevel (), roundsSpentDuringThisFight);
}

Uint8 Battle :: getStatus ()
{ return battleState; }

Sint32 Battle :: getEndurance ()
{ return endurance; }

Uint32 Battle :: getNumberOfVictories ()
{  return numVicts; }

Battle :: ~Battle ()
{
  vector <textMessage *> :: iterator iter;

  iter = tmVec.begin ();
    
  while (iter < tmVec.end ())
    {
      delete (*iter);
      iter ++;
    }

  delete enem;
  delete dw;
  SDL_FreeSurface (bkg);
}

////////////////
// Text Message Supplementary class

textMessage :: textMessage (string & mess, SDL_Color fgCol, SDL_Color * bgCol, Uint32 dur, SDL_Rect & rect, TTF_Font * frigg)
{
  //creates the texture with the given colors and text.
  bk = NULL;

  surf = TTF_RenderText_Solid (frigg, mess.c_str(), fgCol);
  duration = dur;
  pos.x = rect.x;
  pos.y = rect.y;
  done = false;

  if (bgCol != NULL)
    {
      bk = new SDL_Color;
      bk -> r = bgCol -> r;
      bk -> g = bgCol -> g;
      bk -> b = bgCol -> b;
    }

  timer = SDL_GetTicks ();
}

textMessage :: textMessage (string & mess, Uint32 colIndex, TTF_Font * font, Sint32 posx, Sint32 posy, Uint32 waitTime)
{
  //creates the texture with the given colors and text.
  SDL_Color fg;

  bk = NULL;

  pos.x = posx;
  pos.y = posy;

  switch (colIndex)
    {
    case (0): //deal Damage
      bk = new SDL_Color;
      fg.r = 128; fg.g = 0; fg.b = 0;
      bk -> r = 224; bk -> g = 192; bk -> b = 192;
      break;
    case (1): //level change!
      bk = new SDL_Color;
      fg.r = 196; fg.g = 196; fg.b = 196;
      bk -> r = 16; bk -> g = 16; bk -> b = 16;
      break;
    case (2): //Gain HP
      bk = new SDL_Color;
      fg.r = 0; fg.g = 128; fg.b = 0;
      bk -> r = 0; bk -> g = 64; bk -> b = 0;
      break;
    case (3): //Oh NOES, a Monster!
      bk = new SDL_Color;
      fg.r = 192; fg.g = 192; fg.b = 192;
      bk -> r = 0; bk -> g = 0; bk -> b = 64;
      break;
    default:
      bk = NULL;
      fg.r = 0; fg.g = 0; fg.b = 0;
      break;
    }
  
  surf = TTF_RenderText_Solid (font, mess.c_str(), fg);
  duration = waitTime;
  done = false;

  timer = SDL_GetTicks ();
}

void textMessage :: render (SDL_Surface * dest)
{
  Uint32 timeE = SDL_GetTicks () - timer;
  Sint32 halfD = duration / 2;
  Sint32 trans;
  Uint8 transAns;

  //set transparency level
  if (timeE < (duration / 2))
    trans = 192;
  else if (timeE < duration)
    {
      //linear interpolation to 0. Makes the box fade out over time.
      trans = timeE - halfD;
      trans *= -192;
      trans /= halfD;
      trans += 192;      

      pos.y --; //gradually lift the box.
    }
  else
    {trans = 0; done = true;}

  if (timeE >= (duration / 4))
    {
      pos.y --; //gradually lift the box.
    }

  if (trans > 192)
    transAns = 192;
  else if (trans < 0)
    transAns = 0;
  else
    transAns = Uint8(trans);

  SDL_SetAlpha(surf, SDL_SRCALPHA | SDL_RLEACCEL, transAns);

  //draw the box first.
  if (bk != NULL)
    {
      filledEllipseRGBA(dest, pos.x + (surf -> w/2), pos.y + (surf -> h/2),
		      (surf -> w/2) + 2, (surf -> h/2) + 1, bk -> r, bk -> g,
		      bk -> b, transAns);
    }

  SDL_BlitSurface(surf, NULL, dest, &pos);
}

textMessage :: ~ textMessage ()
{
  SDL_FreeSurface (surf);
  delete bk;
}
