#ifndef BATTLE_H
#define BATTLE_H

#include "SDL.h"

#include "DrawWindow.h"
#include "InputObj.h"
#include "UserInput.h"
#include "SDL_ttf.h"
#include "Combatant.h"
#include "Enemy.h"
#include "StatRecord.h"

#include <iostream>
#include <ostream>
#include <sstream>
using namespace std;

//Battle is the part of the game that handles the line-drawing conflict.
//Main sends the necessary information (who is involved, scenario, etc etc.)
//Battle processes the fight, and updates the end result.
//Main calls on Battle for end fight stats, and then reupdates it.

class textMessage {
 public:
  textMessage (string &, SDL_Color, SDL_Color *, Uint32, SDL_Rect &, TTF_Font *);
  textMessage (string &, Uint32, TTF_Font *, Sint32, Sint32, Uint32 = 1500);
  void render (SDL_Surface *);
  ~textMessage ();

  bool done;

 private:
  SDL_Surface * surf;
  SDL_Color * bk;
  SDL_Rect pos;
  Uint32 duration;
  Uint32 timer;
};

class Battle {
 public:
  Battle ();

  void reset ();
  void init (Combatant *, TTF_Font *);

  void render (SDL_Surface *);
  void renderStats (SDL_Surface *, SDL_Color &);
  void renderCombatants (SDL_Surface *);
  void renderBattleIntro (SDL_Surface *, SDL_Color &, SDL_Rect &);
  void renderBattleExtro (SDL_Surface *, SDL_Color &, SDL_Rect &);
  void renderEndurance (SDL_Surface *, SDL_Color &, SDL_Color &, SDL_Rect &);
  void renderResults (SDL_Surface *, SDL_Color &, SDL_Rect &);
  void renderTextMessages (SDL_Surface *);

  void gameLoop ();
  void processInput (const SDL_Event & mainEvent);

  void addMessage (textMessage *);
  void addMessage (string &, SDL_Color, SDL_Color *, Uint32, SDL_Rect &);

  Uint8 getStatus ();

  ~Battle ();
 protected:
  Sint32 getEndurance ();
  Uint32 getNumberOfVictories ();

  bool ifPlayerAttIsGreaterThanEnemyDef ();

  void switchTurns ();
  void addVictory ();
  void startOfRound ();
  void endOfRound ();
  void startOfFight ();
  void endOfFight ();
  bool introIsOver ();
  bool extroIsOver ();
  void degradeStats ();

  void addStats1 ();
  void addStats2 ();

  void animateAttack (Combatant *, Combatant *, Uint32);

  SDL_Surface * textOutput;
  TTF_Font * font;
  SDL_Surface * bkg;
  ostringstream oss;
  
  DrawWindow * dw;
  Enemy * enem;
  Combatant * leftCombatant;
  Combatant * rightCombatant;
  Combatant * currentCombatant;

  Sint32 endurance;
  Uint8 battleState;
  SDL_Color drawRules;

  Uint32 lastTicks;
  Uint8 overkillMode;

  bool countVict;
  bool countLoss;

  Uint32 numVicts;
  Uint32 numHits;
  Uint32 totalDmg;
  Uint32 roundCount;
  Uint32 roundsSpentDuringThisFight;
  Uint32 numAttacks;

  vector <textMessage *> tmVec;

  SDL_Rect * enemyStruct;

  StatRecord stats;
};

#endif
