#include "Combatant.h"

#include "SDL.h"
#include "SDL_ttf.h"
#include "SDL_gfxPrimitives.h"

#include <vector>
#include <string>
#include <iostream>
#include <ostream>
#include <sstream>
using namespace std;

#include "InputObj.h"
#include "MediaPlayer.h"

Combatant :: Combatant ()
{
  name = NULL;
  mo = NULL;
  maxHp = 100;
  hp = 80;
  playerSide = true;
  areYouDead = false;
  io = NULL;

  atkIncreaser = 20;
  atkStyle = 0;
  atkInitLvl = 0;
  atkInitToNext = 100;

  atkCurrent = 0;
  atkLevel = 0;
  atkNext = getCostOfLevel (0, atkStyle, atkIncreaser, atkInitToNext);

  defIncreaser = 20;
  defStyle = 0;
  defInitLvl = 0;
  defInitToNext = 100;

  defCurrent = 0;
  defLevel = 0;
  defNext = getCostOfLevel (0, defStyle, defIncreaser, defInitToNext);

  healIncreaser = 5;
  healStyle = 0;
  healInitLvl = 0;
  healInitToNext = 10;

  healCurrent = 0;
  healLevel = 0;
  healNext = getCostOfLevel (0, healStyle, healIncreaser, healInitToNext);
}

void Combatant :: setInput (InputObj * i)
{ io = i; }

void Combatant :: setSide (bool b)
{ playerSide = b; }

void Combatant :: processInput (const SDL_Event & mainEvent)
{
  if (io != NULL)
    io -> processInput (mainEvent);
}

void Combatant :: render (SDL_Surface * dest, TTF_Font * font)
{
  //draw character
  SDL_Rect picPos;

  //draw the character (should be changed once animation is decided.)

  if (playerSide)
    picPos.x = 85;
  else
    picPos.x = 555;
  
  picPos.y = 280;
  mo -> render (dest, picPos);

  //and now, for text.
  SDL_Color textCol;
  textCol.r = 255;
  textCol.g = 255;
  textCol.b = 255;

  SDL_Rect textPosa;
  if (playerSide)
    textPosa.x = 20;
  else
    textPosa.x = 490;
  textPosa.y = 350;
  textOutput = TTF_RenderText_Solid (font, name -> c_str(), textCol);
  SDL_BlitSurface(textOutput, NULL, dest, &textPosa);
  SDL_FreeSurface (textOutput);

  //draw hp bar here
  SDL_Rect hpBarDim;
  if (playerSide)
    hpBarDim.x = 10;
  else
    hpBarDim.x = 480;

  hpBarDim.y = 330;
  hpBarDim.w = 150;
  hpBarDim.h = 20;

  SDL_Color hpBarCol;
  hpBarCol.r = 96;
  hpBarCol.g = 32;
  hpBarCol.b = 32;

  boxRGBA(dest, hpBarDim.x, hpBarDim.y, hpBarDim.x + hpBarDim.w, hpBarDim.y + hpBarDim.h, 32, 32, 32, 128);

  if (playerSide)
    hpBarDim.x = 10;
  else
    hpBarDim.x = 380;
  hpBarDim.w = getHp () * 150 / getMaxHp ();

  if (!playerSide)
    hpBarDim.x = 630 - hpBarDim.w;

  boxRGBA(dest, hpBarDim.x, hpBarDim.y, hpBarDim.x + hpBarDim.w, hpBarDim.y + hpBarDim.h, 96, 32, 32, 255);

  hpBarDim.w = 150;

  if (!playerSide)
    hpBarDim.x = 630 - hpBarDim.w;

  hpBarCol.r = getHp () * 255 / getMaxHp ();
  rectangleRGBA(dest, hpBarDim.x, hpBarDim.y,
		hpBarDim.x + hpBarDim.w, hpBarDim.y + hpBarDim.h,
		hpBarCol.r, hpBarCol.r, hpBarCol.r, 255);

  if (!playerSide)
    hpBarDim.x = 610;

  textPosa.y = 330;

  ostringstream oss (ostringstream::out);
  oss << int(getHp ());
  textOutput = TTF_RenderText_Solid (font, oss.str ().c_str (), textCol);
  SDL_BlitSurface(textOutput, NULL, dest, &textPosa);
  SDL_FreeSurface (textOutput);
}

bool Combatant :: takeDamage (Uint32 & totalDmg, Uint32 numHits, Uint32 eAtt)
{
  bool overkill = false;
  Sint32 lvlDiff = eAtt - defLevel;

  lvlDiff = eAtt - defLevel;

  if (lvlDiff >= 4)
    totalDmg *= 4;
  else if (lvlDiff == 3)
    totalDmg = totalDmg * 2;
  else if (lvlDiff == 2)
    totalDmg = totalDmg * 7 / 4;
  else if (lvlDiff == 1)
    totalDmg = totalDmg * 3 / 2;
  else if (lvlDiff == 0)
    totalDmg = totalDmg;
  else if (lvlDiff == -1)
    totalDmg = totalDmg * 3 / 4;
  else if (lvlDiff == -2)
    totalDmg = totalDmg / 2;
  else if (lvlDiff == -3)
    totalDmg = totalDmg / 4;
  else
    totalDmg = totalDmg / 8;

  //1) Hit as many dots equal to their defense.
  //Otherwise, deal half.
  if (!playerSide && numHits >= defLevel)
    totalDmg /= 2;

  if (totalDmg >= hp)
    {
      hp = 0; areYouDead = true;
      overkill = true;
      animate (3, 0);
    }
  else if (totalDmg == 0)
    {} //do a laughing pose?
  else
    {
      hp -= totalDmg;
    }
  return overkill;
}

Uint8 Combatant :: emptyHealLevels ()
{
  Uint8 ans = healLevel;

  addHP (healLevel);
  healNext = healInitToNext;
  healCurrent = 0;
  healLevel = 0;
  return ans;
}

Uint32 Combatant :: getHp ()
{ return hp; }

Uint32 Combatant :: getMaxHp ()
{ return maxHp; }

InputObj * Combatant :: getIo ()
{ return io; }

bool Combatant :: getAreYouDead ()
{
  return areYouDead;
}

void Combatant :: animate (Uint32 index, Uint32 subIndex, Combatant * animPartner, Uint32 partnerAnim, Uint32 partnerSubAnim)
{
  //changes animation.
  if (mo != NULL)
    {
      if (animPartner == NULL)
	mo -> changeAnim (index, subIndex);
      else
	mo -> changeAnim (index, subIndex, animPartner -> getMediaPlayer (), partnerAnim, partnerSubAnim);
    }
}

MediaPlayer * Combatant :: getMediaPlayer ()
{
  return mo;
}

string Combatant :: getName ()
{
  return *name;
}

void Combatant :: setName (string & newName)
{
  if (name == NULL)
    name = new string ();

  name -> assign (newName);
}

void Combatant :: addMaxHP (Uint32 mp)
{
  maxHp += mp;

  hp = maxHp;
}

void Combatant :: setMaxHP (Uint32 mp)
{
  maxHp = mp;
  if (hp > maxHp)
    hp = maxHp;
}

void Combatant :: addHP (Uint32 h)
{
  if (h + hp > maxHp)
    hp = maxHp;
  else
    hp += h;
}

void  Combatant :: setMedia (MediaPlayer * m)
{
  mo = m;
}

Sint32 Combatant :: addAtkPoints (Sint32 amnt)
{
  Uint32 initLvl = atkLevel;
  
  atkCurrent += amnt;

  getSkillStats (atkCurrent, atkNext, atkLevel, 0);

  if (initLvl < atkLevel)
    return 1;
  else if (initLvl > atkLevel)
    return -1;
  else
    return 0;
}

void Combatant :: getAtkStats (Uint32 * lvlRet, Sint32 * currRet, Uint32 * toNextRet)
{
  if (currRet != NULL)
    *currRet = atkCurrent;
  if (lvlRet != NULL)
    *lvlRet = atkLevel;
  if (toNextRet != NULL)
    *toNextRet = atkNext;
}

Uint32 Combatant :: getAtkLevel ()
{ return atkLevel; }

void Combatant :: loseAtkLevel (Uint8 numlvls)
{
  Sint32 cost;

  if (atkLevel > numlvls)
    {
      cost = getCostOfLevel (atkLevel - numlvls, atkStyle, atkIncreaser, atkInitToNext);
      atkCurrent -= cost;
      getSkillStats (atkCurrent, atkNext, atkLevel, 0);
    }
  else
    {
      atkLevel = 1;
      atkCurrent = 0;
      atkNext = getCostOfLevel (atkLevel, atkStyle, atkIncreaser, atkInitToNext);
    }

  if (atkLevel < 1)
    atkLevel = 1;
}

void Combatant :: setAtkLevel (Uint8 numlvls)
{
  atkLevel = numlvls;
  atkNext = getCostOfLevel (numlvls, atkStyle, atkIncreaser, atkInitToNext);
  atkCurrent = 0;
}

Sint32 Combatant :: addDefPoints (Sint32 amnt)
{
  Uint32 initLvl = defLevel;

  defCurrent += amnt;
  getSkillStats (defCurrent, defNext, defLevel, 1);

  if (initLvl < defLevel)
    return 1;
  else if (initLvl > defLevel)
    return -1;
  else
    return 0;
}
 
void Combatant :: getDefStats (Uint32 * lvlRet, Sint32 * currRet, Uint32 * toNextRet)
{
  if (currRet != NULL)
    *currRet = defCurrent;
  if (lvlRet != NULL)
    *lvlRet = defLevel;
  if (toNextRet != NULL)
    *toNextRet = defNext;
}

Uint32 Combatant :: getDefLevel ()
{ return defLevel; }

void Combatant :: loseDefLevel (Uint8 numlvls)
{
  Sint32 cost;

  if (defLevel > numlvls)
    {
      cost = getCostOfLevel (defLevel - numlvls, defStyle, defIncreaser, defInitToNext);
      defCurrent -= cost;

      getSkillStats (defCurrent, defNext, defLevel, 1);
    }
  else
    {
      defLevel = 0;
      defCurrent = 0;
      defNext = getCostOfLevel (defLevel, defStyle, defIncreaser, defInitToNext);
    }
}

void Combatant :: setDefLevel (Uint8 numlvls)
{
  defLevel = numlvls;
  defNext = getCostOfLevel (defLevel, defStyle, defIncreaser, defInitToNext);
  defCurrent = 0;
}

Sint32 Combatant :: addHealPoints (Sint32 amnt)
{
  Uint32 initLvl = healLevel;

  healCurrent += amnt;
  getSkillStats (healCurrent, healNext, healLevel, 2);

  if (initLvl < healLevel)
    return 1;
  else if (initLvl > healLevel)
    return -1;
  else
    return 0;
}

void Combatant :: getHealStats (Uint32 * lvlRet, Sint32 * currRet, Uint32 * toNextRet)
{
  if (currRet != NULL)
    *currRet = healCurrent;
  if (lvlRet != NULL)
    *lvlRet = healLevel;
  if (toNextRet != NULL)
    *toNextRet = healNext;
}

Uint32 Combatant :: getHealLevel ()
{ return healLevel; }

Sint32 Combatant :: getCostOfLevel (Sint32 lvl, Uint8 increaseStyle, Uint8 increaser, Uint8 init)
{
  Sint32 ans;

  switch (increaseStyle)
    {
    case (0):
      //linear
      ans = init + (lvl * increaser);
      break;
    case (1):
      //exponential (it's out of 16, remember.)
      if (lvl != 0)
	ans = init * lvl * increaser / 16;
      else
	ans = init;
      break;
    default:
      break;
    }

  return ans;
}

void Combatant :: getSkillStats (Sint32 & curr, Uint32 & next, Uint32 & lvl, Uint32 index)
{
  //increase levels.
  while (curr >= Sint32(next))
    {
      curr -= next;
      lvl += 1;
      switch (index)
	{
	case (0): next = getCostOfLevel (lvl - atkInitLvl, atkStyle, atkIncreaser, atkInitToNext); break;
	case (1): next = getCostOfLevel (lvl - defInitLvl, defStyle, defIncreaser, defInitToNext); break;
	case (2): next = getCostOfLevel (lvl - healInitLvl, healStyle, healIncreaser, healInitToNext); break;
	default: break;
	}
    }

  //decrease levels.
  while (curr < 0 && lvl > 0)
    {
      lvl -= 1;

      switch (index)
	{
	case (0): next = getCostOfLevel (lvl - atkInitLvl, atkStyle, atkIncreaser, atkInitToNext); break;
	case (1): next = getCostOfLevel (lvl - defInitLvl, defStyle, defIncreaser, defInitToNext); break;
	case (2): next = getCostOfLevel (lvl - healInitLvl, healStyle, healIncreaser, healInitToNext); break;
	default: break;
	}

      curr += next;
    }
  if (curr < 0)
    curr = 0;
}

Uint32 Combatant :: numberOfPossibleAnims (Uint32 animCat)
{
  return mo -> numberOfPossibleAnims (animCat);
}

Combatant :: ~Combatant ()
{
  delete name;
}
