#include "DrawWindow.h"
#include "LinePattern.h"

#include "SDL.h"
#include "SDL_gfxPrimitives.h"
#include "InputObj.h"
#include "Combatant.h"
#include "Enemy.h"

#include <iostream>
using namespace std;

DrawWindow :: DrawWindow ()
{
  dimensions.x = 170;
  dimensions.y = 30;
  dimensions.w = 300;
  dimensions.h = 300;  
  lp = new LinePattern;
  drawingState = 0;
  attStruct = NULL;
  lastTicks = SDL_GetTicks ();
  lastTicksDiscrete = SDL_GetTicks ();
  playerC = NULL;
  hitsDuringPattern = 0;
}

void DrawWindow :: init (Combatant * c, TTF_Font * f, Enemy * e, Uint32 numPats, Uint32 numPts)
{
  drawingState = 0;
  playerC = c;
  font = f;

  enduranceLossRate = e -> getEnduranceDrain ();
  patternSizeMult = e -> getPatternSizeMultiplier ();
  patternDiff = e -> getPatternDifficultyOffset ();
  numExtraPatterns = numPats;
  numExtraPts = numPts;
  hitsDuringPattern = 0;
}

void DrawWindow :: setEnduranceLossRate (Uint32 e)
{
  enduranceLossRate = e;
}

Uint32 DrawWindow :: getEnduranceLossRate ()
{
  return enduranceLossRate;
}

void DrawWindow :: clear ()
{
  drawingState = 0;
}

void DrawWindow :: render (SDL_Surface * dest, Uint8 overkillMode)
{
  boxRGBA(dest, dimensions.x, dimensions.y, dimensions.x + dimensions.w, dimensions.y + dimensions.h, 192, 192, 192, 32);

  lp -> render (dest, dimensions, drawingState, overkillMode);
}

void DrawWindow :: setLinePattern (LinePattern * llp)
{
  lp = llp;
}

void DrawWindow :: drawEvaluator (InputObj * io, Sint32 & end, Uint32 & numHits, Uint8 & overkillMode, Uint32 enemDef, Uint32 pAtt)
{
  //depending on the state:
  //0: Generate 1 point.
  //1: Wait for the user to click on the point.
  //2: Generate patterns using the current settings.
  //3: The user is drawing. Collect statistics.
  //4: If out of endurance, the round is over.
  //5: If out of patterns, evaluate the scene and goto 2.

  Uint8 status;
  Uint32 enduranceLostDiscrete;
  Sint32 dmgDiff;

  switch (drawingState)
    {
    case (0):
      //0: Generate 1 point.
      //generate the point
      lp -> setInitialPatterns (dimensions);
      drawingState = 1;
      break;
    case (1):
      //1: Wait for the user to click on the point.
      if (lp -> checkOnFirstPoint (io, dimensions) == 1)
	drawingState = 2;
      break;
    case (2):
      //2: Generate patterns using the current settings.
      dmgDiff = enemDef - pAtt;

      if (lp -> generatePatterns (io, dimensions, overkillMode, patternSizeMult, numExtraPts, patternDiff, numExtraPatterns, dmgDiff) == 1)
	drawingState = 3;

      lastTicks = SDL_GetTicks ();
      lastTicksDiscrete = SDL_GetTicks ();
      enduranceLost = 0;
      hitsDuringPattern = 0;
      break;
    case (3):
      //3: The user is drawing. Collect statistics.
      status = lp -> checkOnPoints (io, dimensions, numHits, hitsDuringPattern);

      if (overkillMode != 2 && hitsDuringPattern >= enemDef)
	overkillMode = 1;

      enduranceLostDiscrete = (SDL_GetTicks () - lastTicksDiscrete) * enduranceLossRate / 1000;
      lastTicksDiscrete = SDL_GetTicks ();

      if (status == 1) //out of patterns to draw
	{
	  drawingState = 5; hitsDuringPattern = 0;

	  if (overkillMode != 2)
	    overkillMode = 0;
	}
      else if (Sint32 (enduranceLostDiscrete) >= end || end <= 0) //out of endurance
	{
	  drawingState = 4; hitsDuringPattern = 0;
	  if (overkillMode != 2)
	    overkillMode = 0;
	}
      end -= enduranceLostDiscrete;
      enduranceLost += enduranceLostDiscrete;
      break;
    case (4):
      //4: If out of endurance, the round is over.
      end = 0;
      break;
    case (5):
      //5: If out of patterns, evaluate the scene and goto 2.
      if (overkillMode != 2)
	evaluateRound (numHits, end, overkillMode);
      drawingState = 2;
      break;
    default: break;
    }
}

void DrawWindow :: evaluateRound (Uint32 & numHits, Sint32 & end, Uint8 overkillmode)
{
  //evaluates the given statistics and makes an attack.
  Uint32 damage;
  Uint32 playerCAttLvl;
  playerC -> getAtkStats (&playerCAttLvl, NULL, NULL);

  //do something with playerCAttLvl here...
  damage = numHits;

  //return endurance
  if (overkillmode != 2)
    end += enduranceLost / 4;

  SDL_Event attMade;

  delete attStruct;
  attStruct = new SDL_Rect ();
  attStruct -> w = 0;
  attStruct -> h = damage;
  attStruct -> x = numHits;

  attMade.type = SDL_USEREVENT;
  attMade.user.code = 0;
  attMade.user.data1 = (void *) attStruct;
  attMade.user.data2 = 0;
  SDL_PushEvent(&attMade);

  numHits = 0;
}

SDL_Rect DrawWindow :: getDimensions ()
{
  return dimensions;
}

int DrawWindow :: getDrawingStatus ()
{ return drawingState; }

DrawWindow :: ~DrawWindow ()
{
  delete lp;
  delete attStruct;
}
