#include "Enemy.h"
#include <math.h>

#include "SDL.h"

#include <vector>
#include <string>

#include "sdMath.h"

using namespace std;

#define numEnemyTypes 6

Enemy :: Enemy ()
{
  //load media resource now!
  string fn2 ("demoBadguy");
  mr.resize (0);

  mr.push_back (new MediaResource (fn2));

  c = NULL; mp = NULL;

  setUpBadguy (0);
}

void Enemy :: setUpBadguy (Uint32 numVicts)
{
  //creates an enemy, based on the number of victories.

  if (c != NULL)
    delete c;
  c = new Combatant;

  string name ("BadGuy");

  enemyTypeIndex = randomUniformInt (numEnemyTypes);

  //can't wait to delete this...
  c -> setSide (false);

  vector <string> hi;
  hi.resize (5);

  //determine enemy type here. It should set up the names and the stats for the enemy.

  //enemyTypeIndex = 10;

  switch (enemyTypeIndex)
    {
    case (0):
      //slime: slimes have a lot of HP and big patterns, with low defense.
      //Accuracy is key here.
      //slimes really wear you down.
      hi.at(0).assign ("Slime");
      hi.at(1).assign ("Gooey");
      hi.at(2).assign ("Gelatinous (Bear) Cub");
      hi.at(3).assign ("GAAAAK!");
      hi.at(4).assign ("Cosby");

      getStatsForBadguy (numVicts, hi, 60, 1.3, 0.7, 0.2, 0.7, 0.5);
      break;
    case (1):
      //fairy. To beat quickly, you need to be grab as many lines as possible. They don't have high HP, luckily.
      //fairies are difficult because it's hard to squeeze out good damage output.
      hi.at(0).assign ("Fairy");
      hi.at(1).assign ("Nymph (dressed)");
      hi.at(2).assign ("Pretty! Pretty!");
      hi.at(3).assign ("Trendy Titania");
      hi.at(4).assign ("Mrs. Sparkle");
      getStatsForBadguy (numVicts, hi, 15, 1.0, 1.0, 0.5, 0.5, 1.5);
      break;
    case (2):
      //Armored: High Defense and Offense.
      //The punisher of bad stats! This guy punishes you for near-kills! Finish him fast!
      hi.at(0).assign ("Dark (crusader) Knight");
      hi.at(1).assign ("Golem");
      hi.at(2).assign ("Treant (Metal?)");
      hi.at(3).assign ("Gigantousant");
      hi.at(4).assign ("McBamf");
      getStatsForBadguy (numVicts, hi, 24, 1.5, 1.5, 0.2, 0.3, 0.3);
      break;
    case (3):
      //Wizard: Little time, and high damage. Finish him fast!
      //Very punishing in late game!
      hi.at(0).assign ("Mage");
      hi.at(1).assign ("Archiemage");
      hi.at(2).assign ("Smarter-than-thou");
      hi.at(3).assign ("Zappity Doom!");
      hi.at(4).assign ("Wizeard");
      getStatsForBadguy (numVicts, hi, 21, 1.7, 1.0, 0.5, 0.4, 0.5);
      break;
    case (4):
      //Thief: Annoying Patterns, high def
      //You must draw their patterns! Unlike the fairies, whom you can skip some as long as you hit quickly...
      hi.at(0).assign ("Thief");
      hi.at(1).assign ("'Treasure Hunter'");
      hi.at(2).assign ("Entrepeneur");
      hi.at(3).assign ("Longarms");
      hi.at(4).assign ("The CEO");
      getStatsForBadguy (numVicts, hi, 21, 1.0, 1.7, 0.3, 0.3, 1.0);
      break;
    case (5):
      //Dragon: High HP, Att, Size
      //he toughest enemy of them all!...or something. Should be annoying, here to chop you down a peg. Just the bastard enemy.
      hi.at(0).assign ("Dragon");
      hi.at(1).assign ("Bling Bling Beast");
      hi.at(2).assign ("Princess Magnet");
      hi.at(3).assign ("Big Mama!");
      hi.at(4).assign ("O, teh Ancient One!!1!");
      getStatsForBadguy (numVicts, hi, 30, 1.3, 1.3, 0.3, 0.6, 0.7);
      break;      
    default:
      //This is the test enemy. Balance is not a factor.
      hi.at(0).assign ("BadGuy");
      hi.at(1).assign ("ReallyBAD!");
      hi.at(2).assign ("Ohmygosh");
      hi.at(3).assign ("DOOMED");
      hi.at(4).assign ("Oh boy...");

      getStatsForBadguy (numVicts, hi, 10, 1.5, 2.0, 0.35, 0.5, 0.5);
      break;
    }

  //should point to the correct enemy type
  if (mp != NULL)
    delete mp;
  mp = new MediaPlayer (mr.at (0));
  c -> setMedia (mp);
}

void Enemy :: getStatsForBadguy (Uint32 powerlvl, vector <string> names, Uint32 hpMult, double attMult, double defMult, double patternDiffMult, double sizeMult, double enduranceMult)
{
  //gets stats for the badguy. The multiplication parameters are sent by the badguy.
  //powerChange ranges from 1.0 to 7.0, min to max.

  double powerChange = 1.0;
  Uint32 whole = (powerlvl) / 5, part = (powerlvl) % 5;
  double rand;

  //edit powerChange by powerlvl.
  //It should increase quickly and then drop after a few matches.
  //Expecting it to max after 25 matches or so.

  if (powerlvl > 25)
    {whole = 5; part = powerlvl - 25;}
  
  //power change should go from x -> x+2, then fall to x+1 over the course of 5 matches.
  powerChange = double (whole) + (double (part) * .5);

  if (powerChange < .5)
    powerChange = .5;

  //HP: initial + bonus per level.
  Uint32 hp;
  hp = Uint32 (powerChange * hpMult);
  rand = (randomUniform () - .5) * .2;
  rand += 1.0;
  hp = Uint32 (rand * hp);
  hp += 50;

  c -> setMaxHP (hp);
  c -> addHP (hp);

  //Attack: linearly add.
  double alvl = (powerChange * attMult);

  double frac, dec;
  rand = randomUniform ();

  frac = modf (alvl, &dec);

  if (rand < frac)
    alvl += 1;

  if (alvl < 0.0)
    alvl = 0.0;

  c -> setAtkLevel (Uint32(alvl));

  //Defense: linearly add.
  double dlvl = powerChange * defMult;
  rand = randomUniform ();

  frac = modf (alvl, &dec);

  if (rand < frac)
    dlvl += 1;

  if (dlvl < 0)
    dlvl = 0;

  c -> setDefLevel (Uint32(dlvl));

  //patternDiff: linear
  patternDifficultyOffset = 0.0 + (powerChange * patternDiffMult);

  //size: log.
  //sizeMult = maxsize: .1 = 1.23, .2 = 1.4, .3 = 1.5, .5 = 1.65, 0.75 = 1.8, 1 = 2
  double sizeBase = powerChange * sizeMult;
  if (sizeBase < -.99)
    sizeBase = -.99;

  patternSizeMultiplier = log10 (1.0 + sizeBase);

  if (patternSizeMultiplier < -0.5)
    patternSizeMultiplier = -0.5;

  patternSizeMultiplier += 1.0;

  if (patternSizeMultiplier < 0.0)
    patternSizeMultiplier = 0.1;

  //end lost? linear.
  double endloss = powerChange * enduranceMult;

  if (endloss < 0.0)
    endloss = 0.0;
  if (endloss > 9.0)
    endloss = 9.0;

  enduranceDrain = 200 + Uint32 (33 * endloss);

  //set the name.
  if (whole >= names.size ())
    c -> setName (names.back ());
  else
    c -> setName (names.at (whole));

}

void Enemy :: halvePatternSize ()
{
  patternSizeMultiplier *= .75;
}

Combatant * Enemy :: getCombatant ()
{  return c;}

double Enemy :: getPatternSizeMultiplier ()
{  return patternSizeMultiplier;}

double Enemy :: getPatternDifficultyOffset ()
{  return patternDifficultyOffset;}
 
Uint32 Enemy :: getEnduranceDrain ()
{  return enduranceDrain;}

Uint32 Enemy :: getEnemyIndex ()
{  return enemyTypeIndex;}

Enemy :: ~Enemy ()
{
  delete c;

  //clear media resource now!
  delete mp;

  //vector delete this!
  vector <MediaResource *> :: iterator iter;
  iter = mr.begin ();
  while (iter < mr.end ())
    {
       delete (*iter);
      iter ++;
    }
}
