#include <math.h>
#include "LinePattern.h"

#include "sdMath.h"

#include "SDL.h"
#include "SDL_gfxPrimitives.h"
#include "InputObj.h"
#include <vector>
#include <list>
#include <iostream>
using namespace std;

#define shapeDropoff 0
#define shapeS 1
#define shapeU 2
#define shapeLoop 3
#define shapeSigma 0.3
#define shapeMax 4

#define numPI 3.1415927
#define numPI_2 1.5707963
#define numPI_4 0.78539816


//4 Basis functions for a cubic bezier spline|
#define B4(t)  t * t * t
#define B3(t)  3 * t * t * (1 - t)
#define B2(t)  3 * t * (1 - t) * (1 - t)
#define B1(t)  (1 - t) * (1 - t) * (1 - t)

//numPI is the defined constant.3.141590
//trig functions are in radians.

LinePattern :: LinePattern ()
{
  activePoints = 0;
  originPoint.x = 150;
  originPoint.y = 150;
  easyPoints.resize (0);
  hardPoints.resize (0);
  newPoints.resize (0);
}

void LinePattern :: render (SDL_Surface * dest, SDL_Rect & dim, Uint8 state, Uint8 overkillMode)
{
  if (state == 0) //not ready yet!
    return;

  switch (activePoints)
    {
    case (0): //just draw the initial point.
      renderPoint (dest, dim, &originPoint, 255, overkillMode);
      break;
    case (1):
    case (2):
      renderVector (dest, dim, &easyPoints, overkillMode);
      if (activePoints == 2)
	break;
    case (3):
      renderVector (dest, dim, &hardPoints, overkillMode);
      break;
    default: break;
    }
}

void LinePattern :: renderPoint (SDL_Surface * dest, SDL_Rect & dim, patternPoint * pp, Uint8 intensity, Uint8 overkillMode)
{
  //draws the point on the bitmap.
  if (pp -> type != 0)
    return;

  //just draw the point for now.
  Uint8 r = 0, g = 192, b = 0, a = intensity;
  Uint8 outr = 64, outg = 64, outb = 64;

  if (pp -> side == 0)
    r = 0, g = 192, b = 0;
  else
    r = 192, g = 192, b = 192;

  switch (overkillMode)
    {
    case (0):
      outr = 64, outg = 64, outb = 64;
      break;
    case (1):
      outr = 0, outg = 0, outb = 0;
      break;
    default: outr = 0, outg = 64, outb = 0; break;
    }

  if (pp -> type == 0)
    filledCircleRGBA(dest, pp -> x + dim.x, pp -> y + dim.y, pp -> size + 1, outr, outg, outb, 255);

  //this is where you handle the shape, depending on the bonus.
  if (pp -> type == 0)
    filledCircleRGBA(dest, pp -> x + dim.x, pp -> y + dim.y, pp -> size, r, g, b, a);
}

void LinePattern :: renderLine (SDL_Surface * dest, SDL_Rect & dim, patternPoint * pp, patternPoint * pp2, Uint8 intensity, Uint8 overkillMode)
{
  //draws a line connecting two points on the bitmap.

  Uint8 a = intensity;
  Uint8 outr = 192, outg = 192, outb = 192;

  if (intensity == 0)
    a = 192;
  else
    {
      a = 192 - (intensity * 16);
      if (a < 64)
	a = 64;
    }

  switch (overkillMode)
    {
    case (0):
      outr = 192, outg = 192, outb = 192;
      break;
    case (1):
      if (pp -> bonus == 0)
	outr = a, outg = a, outb = a;
      else if (pp -> bonus == 1)
	outr = a, outg = 0, outb = 0;
      else if (pp -> bonus == 2)
	outr = 0, outg = 0, outb = a;
      break;
    default: outr = 0, outg = a, outb = 0; break;
    }

  if (pp -> type == 2)
    outr = 64, outg = 64, outb = 64;

  //aa
  lineRGBA(dest, dim.x + pp->x, dim.y + pp->y, dim.x + pp2->x, dim.y + pp2-> y, outr, outg, outb, 255);
}

void LinePattern :: renderVector (SDL_Surface * dest, SDL_Rect & dim, vector <patternPoint *> * vec, Uint8 overkillMode)
{
  Uint8 intens = 255, maxIndex, i;

  maxIndex = vec -> size ();

  for (i = 0; i < maxIndex; i ++)
    {
      if (i == 0)
	intens = 64;
      else
	intens = vec -> at (i) -> lengthOfLine;

      if (i < maxIndex - 1)
	renderLine (dest, dim, vec -> at (i), vec -> at (i+1), intens, overkillMode);
    }

  for (i = 0; i < maxIndex; i ++)
    {
      renderPoint (dest, dim, vec -> at (i), intens, overkillMode);
    }
}

void LinePattern :: setInitialPatterns (SDL_Rect & dim)
{
  double rad, theta, vary;

  //create the first point, given the round's seed settings.
  activePoints = 0;
  originPoint.x = randomUniformInt (dim.w);
  originPoint.y = randomUniformInt (dim.h);  
  originPoint.size = 10;
  originPoint.type = 0;
  originPoint.bonus = 0;
  originPoint.side = 0;
  
  //point towards middle...unless you're at the middle
  if (originPoint.x == dim.w/2 && originPoint.y == dim.h/2)
    dir = randomUniform () * 2.0 * numPI;
  else
    {
      rectToPolar (dim.w/2 - originPoint.x, dim.h/2 - originPoint.y, rad, theta);
      //add some randomization there...

      vary = randomUniform () - .5;
      vary *= numPI_4;
      dir = theta + vary;
    }
}

Uint8 LinePattern :: checkOnFirstPoint (InputObj * io, SDL_Rect & dim)
{
  //using the input, sees if the first point has been clicked on.
  //if so, returns 1.

  if (mouseOverPoint (io, &originPoint, dim))
    return 1;

  return 0;
}

Uint8 LinePattern :: checkOnPoints (InputObj * io, SDL_Rect & dim, Uint32 & numHits, Uint32 & numHitsDuringPattern)
{
  //using the input, sees if points have been clicked on.
  //determines if the user is going on the easy or hard path, and disables the other.
  //if it has reached the end of the path, returns 1.

  vector <patternPoint *> * vec = NULL;
  bool ans = false;

  if (activePoints == 1 || activePoints == 2)
    {
      //check easyPoints
      vec = &easyPoints;
      ans = checkVec (io, dim, vec, numHits, numHitsDuringPattern);
    }

  if (activePoints == 1 || activePoints == 3)
    {
      //check hardPoints
      vec = &hardPoints;
      ans = checkVec (io, dim, vec, numHits, numHitsDuringPattern);
    }
    
  if (ans)
    return 1;

  return 0;
}

bool LinePattern :: checkVec (InputObj * io, SDL_Rect & dim, vector <patternPoint *> * vec, Uint32 & numHits, Uint32 & numHitsDuringPattern)
{
  //returns true if the user has reached the end of the vector.
  vector <patternPoint *> :: iterator iter;
  vector <patternPoint *> :: iterator iter2;
  bool endOfPattern = false;
  iter = vec -> begin ();
  while (iter < vec -> end ())
    {
      if (mouseOverPoint (io, *iter, dim))
	{
	  //add to statistics
	  numHits ++;
	  numHitsDuringPattern++;

	  if ((*iter) -> bonus != 0)
	    {
	      //send an event to alert Battle that you've gained stats!
	      lpEvent.type = SDL_USEREVENT;
	      lpEvent.user.code = 1;
	      statBoosted = (*iter) -> bonus;
	      statBoostedAmnt = 1; //maybe make it depend on the size?
	      lpEvent.user.data1 = (void *) statBoosted;
	      lpEvent.user.data2 = (void *) statBoostedAmnt;
	      
	      SDL_PushEvent(&lpEvent);
	    }

	  //check end of pattern first.
	  //if you hit one of the big ones, the pattern is over.
	  if ((*iter) -> type == 0)
	    endOfPattern = true;

	  //mark previous entries as blank
	  
	  //iter2 = vec -> begin ();
	  iter2 = iter;

	  while (iter2 <= iter)
	    {
	      (*iter2) -> type = 2;
	      (*iter2) -> size = 2;
	      iter2 ++;
	    }

	  //switch activePoints: is this the end of the pattern?	  
	  if (vec == &easyPoints)
	    activePoints = 2;
	  else if (vec == &hardPoints)
	    activePoints = 3;

	  if (endOfPattern)
	    return true;
	}
      iter ++;
    }
  return false;
}

Uint8 LinePattern :: generatePatterns (InputObj * io, SDL_Rect & dim, Uint8 & overkillMode, double patSize, Uint32 numExtraPts, double patternOffset, Uint32 numPatterns, Sint32 dmgDiff)
{
  //creates patterns using the already loaded data.
  //returns 1.

  patternPoint * pp = NULL;
  vector <patternPoint *> :: iterator iter;

  iter = easyPoints.begin ();
  while (iter < easyPoints.end ())
    {delete (*iter); iter++;}
  easyPoints.clear ();
  easyPoints.resize (0);

  iter = hardPoints.begin ();
  while (iter < hardPoints.end ())
    {delete (*iter); iter++;}
  hardPoints.clear ();
  hardPoints.resize (0);

  newPoints.clear ();
  newPoints.resize (0);

  //OK, to generate patterns: We're using cubic splines (perhaps multiple.)
  //0) set start point away from the mouse
  //1) Decide on the general shape and bonus type.
  //2) Decide on the direction and size.
  //3) For each axis:
  // a) The shape determines the order of the control points 
  // b) Put in 100 length grid, and space accordingly.
  // c) make a spline and interpolate the intermediate points.  
  //5) use angle and scale to size (100% parallel, 50% perpendicular)
  //6) bounce points off of the sides.

  Uint32 shape = shapeDropoff;
  double shapeChooser, shapeDec, shapeFrac;
  Sint32 shapeChooser2, shapeInt;
  Uint32 parSize = 100, perpSize = 50;
  Uint8 side = 1;

  vector <Sint32> parallelPoints;
  vector <Sint32> perpendicPoints;

  vector <Sint32> parallelRange;
  vector <Sint32> perpendicRange;
  Uint8 reward;
  Uint32 attProb = 40;
  Uint32 defProb = 40;
  Uint32 lastPointIndex;

  //4) set start point away from the mouse

  pp = new patternPoint;
  pp -> x = io -> getCursorPosX () - dim.x;
  pp -> y = io -> getCursorPosY () - dim.y;
  pp -> side = side;

  //start loop here
  for (Uint8 i = 0; i < numPatterns; i ++)
    {
      shapeChooser = patternOffset + normalRandom (shapeSigma);
  
      shapeFrac = modf (shapeChooser, &shapeDec);
      shapeInt = Sint32 (shapeDec);

      if (randomUniform () < shapeFrac)
	shapeChooser2 = shapeInt + 1;
      else
	shapeChooser2 = shapeInt;

      if (shapeChooser2 < 0)
	shapeChooser2 = 0;
      if (shapeChooser2 >= shapeMax)
	shapeChooser2 = shapeMax - 1;
      shape = Uint32 (shapeChooser2);

      parallelPoints.resize (0);
      perpendicPoints.resize (0);

      //1) Decide on the general shape and bonus type.
      //2) Decide on the direction and size.
      loadShape (parallelRange, perpendicRange, shape, pp, parSize, perpSize);

      //randomize the rewards
      reward = randomUniformInt (100);

      if (reward < attProb)
	pp -> bonus = 1;
      else if (reward < attProb + defProb)
	pp -> bonus = 2;
      else
	pp -> bonus = 0;

      //give health if in overkill mode
      if (overkillMode == 2)
	pp -> bonus = 3;
  
      //3) For each axis:
      // a) The shape determines the order of the control points 
      // b) Put in 100 length grid, and space accordingly.
      // c) make a spline and interpolate the intermediate points.

      //You must call this function for BOTH vectors, or the program will crash in addPatternPoints.
      getSplinePoints (parallelPoints, parallelRange, numExtraPts);
      getSplinePoints (perpendicPoints, perpendicRange, numExtraPts);

      //5) use angle and scale to size (100% parallel, 50% perpendicular)
      //6) bounce points off of the sides.
      if (i == 0)
	lastPointIndex = addPatternPoints (newPoints, parallelPoints, perpendicPoints, pp, dim, parSize, perpSize, patSize, io, false);
      else
	lastPointIndex = addPatternPoints (newPoints, parallelPoints, perpendicPoints, pp, dim, parSize, perpSize, patSize, io, true);

  //set pp to the new end (add distance, too!)
  pp -> x = newPoints.back () -> x;
  pp -> y = newPoints.back () -> y;
    }
  //end loop here

  //7) Copy the new points onto the current vector.
  iter = newPoints.begin ();

  while (iter < newPoints.end ())
    {
      easyPoints.push_back (*iter);
      iter++;
    }

  //8) Set the end vectors' sizes and such...unless the last altered point is already too close to the end.
  if (lastPointIndex < easyPoints.size () * 4 / 5)
    lastPointIndex = easyPoints.size () - 1;

  easyPoints.at (lastPointIndex) -> size = 3;
  easyPoints.at (lastPointIndex) -> type = 0;

  //if (overkillMode != 2)
  //easyPoints.at (lastPointIndex) -> bonus = 0;

  delete pp;
  //when you're done with generating points...
  activePoints = 1; //draw all points.

  return 1;
}

void LinePattern :: loadShape (vector <Sint32> & parallel, vector <Sint32> & perp, Uint32 shape, patternPoint * pp, Uint32 & parSize, Uint32 & perpSize)
{
  //based on the shape, this creates the ordering for the parallel and perpendicular direction points.
  //This also sets the bonusType and size of the pattern.

  //shape format:
  // 0 is the half-way point of the 2nd pt, where it's expected to be.
  // 1 is the standard deviation width. It's measured in 1/16ths.

  parallel.resize (6);
  perp.resize (6);

  switch (shape)
    {
    case (shapeLoop):
      parallel.at (0) = 80; parallel.at (1) = 0; parallel.at (2) = 80; parallel.at (3) = 0; parallel.at (4) = 0; parallel.at (5) = 0;
      perp.at (0) = 80; perp.at (1) = 0; perp.at (2) = 20; perp.at (3) = 0; perp.at (4) = 100; perp.at (5) = 0;
      parSize = 100; perpSize = 50;
      break;
    case (shapeU):
      parallel.at (0) = 70; parallel.at (1) = 0; parallel.at (2) = 70; parallel.at (3) = 0; parallel.at (4) = 0; parallel.at (5) = 0;
      perp.at (0) = 20; perp.at (1) = 0; perp.at (2) = 80; perp.at (3) = 0; perp.at (4) = 100; perp.at (5) = 0;
      parSize = 100; perpSize = 50;
      break;
    case (shapeS):
      parallel.at (0) = 50; parallel.at (1) = 160; parallel.at (2) = 50; parallel.at (3) = 160; parallel.at (4) = 100; parallel.at (5) = 0;
      perp.at (0) = 30; perp.at (1) = 64; perp.at (2) = 70; perp.at (3) = 64; perp.at (4) = 100; perp.at (5) = 64;
      parSize = 100; perpSize = 50;
      break;
    case (shapeDropoff):
      parallel.at (0) = 40; parallel.at (1) = 0; parallel.at (2) = 80; parallel.at (3) = 0; parallel.at (4) = 100; parallel.at (5) = 0;
      perp.at (0) = 20; perp.at (1) = 0; perp.at (2) = 60; perp.at (3) = 0; perp.at (4) = 100; perp.at (5) = 0;
      parSize = 100; perpSize = 50;
      break;
    default:
      parallel.at (0) = 33; parallel.at (1) = 64; parallel.at (2) = 66; parallel.at (3) = 64; parallel.at (4) = 100; parallel.at (5) = 64;
      perp.at (0) = 0; perp.at (1) = 4; perp.at (2) = 0; perp.at (3) = 4; perp.at (4) = 0; perp.at (5) = 4;
      parSize = 150; perpSize = 50;
      break;
    }
}

void LinePattern :: getSplinePoints (vector <Sint32> & points, vector <Sint32> & range, Uint32 numPtsMult)
{
  //3) For each axis:
  // a) The shape determines the order of the control points 
  // b) Put in 100 length grid, and space accordingly.
  // c) make a spline and interpolate the intermediate points.

  //range has the starting point and size of points 2 3 and 4. 1 is always the origin.
  points.resize (0);

  Sint32 startingPt, posChange;
  double sizeRange;
  Uint8 ptIndex;
  Uint32 numPts = 48 * (numPtsMult / 16);

  Sint32 p0 = 0, p1 = 0, p2 = 0, p3 = 0;

  double t, posD;
  Sint32 posI;
  
  startingPt = range.at (0);
  sizeRange = range.at (1);
  posChange = Sint32(normalRandom (1.0) * double(sizeRange) / 16.0);
  p1 = startingPt + posChange;

  startingPt = range.at (2);
  sizeRange = range.at (3);
  posChange = Sint32(normalRandom (1.0) * double(sizeRange) / 16.0);
  p2 = startingPt + posChange;

  startingPt = range.at (4);
  sizeRange = range.at (5);
  posChange = Sint32(normalRandom (1.0) * double(sizeRange) / 16.0);
  p3 = startingPt + posChange;
  
  //get ready to interpolate!

  //it is imperitave that the points are inserted IN ORDER, since that's how they connect.
  //Extra points are inserted at the (hopefully) correct times.
  points.push_back (p0);

  for (ptIndex = 1; ptIndex <= numPts; ptIndex++)
    {
      //temp1 is the parameter for the spline (ranges from 0 to 1.)
      t = double (ptIndex) / double(numPts);

      posD = p0 * B1(t) + p1 * B2(t) + p2 * B3(t) + p3 * B4(t);

      //temp2 is the interpolated position.
      posI = Sint32 (posD);

      points.push_back (posI);
    }

  points.push_back (p3);
}

Uint32 LinePattern :: addPatternPoints (vector <patternPoint *> & newP, vector <Sint32> & parP, vector <Sint32> & perP, patternPoint * startP, SDL_Rect & dim, Uint8 parSize, Uint8 perpSize, double patSize, InputObj * io, bool ignoreFirst)
{
  //returns the index of the last point created when finished.

  //5) use angle and scale to size
  //6) bounce points off of the sides.

  double theta1, rad, dX, dY, rad2, theta;

  Uint8 bonusT = startP -> bonus, side = startP -> side;

  patternPoint * pp = NULL, pp2;
  Uint8 i = 0;
  Uint32 lasti = 0;
  double distPar, distPerp;
  double dx, dy, possibleX, possibleY;
  Uint32 firstIndex = newP.size ();
  vector <double> distanceBetweenDots;
  double lengthOfPattern, relativeLength;

  distanceBetweenDots.resize (parP.size() - 1);

  pp2.x = startP -> x;
  pp2.y = startP -> y;

  for (i = 0; i < parP.size (); i ++)
    {
      distPar = parP.at(i);
      if (i > 0)
	distPar -= parP.at(i-1);
      distPar *= parSize / 100.0;

      distPerp = perP.at(i);
      if (i > 0)
	distPerp -= perP.at(i-1);
      distPerp *= perpSize / 100.0;

      //translate from pattern space to world space.

      //EXCEPTION: (0,0) is undefined in polar coordinates.
      if (distPar >= 0.001 || distPar <= -0.001 || distPerp >= 0.001 || distPerp <= -0.001)
	{
	  //It's too easy to do this in polar space!
	  rectToPolar (distPar, distPerp, rad, theta1);

	  //rotate the angle by the proper amount
	  theta1 = restrictAnglePos (theta1);
	  theta1 += dir;

	  //OK, results are a comin
	  polarToRect (rad, theta1, dX, dY);
	}
      else
	{dX = 0.0; dY = 0.0;}

      pp = new patternPoint ();
      pp -> x = pp2.x;
      pp -> y = pp2.y;
      pp -> size = 0;

      pp -> type = 1;

      pp -> bonus = bonusT;
      pp -> side = side;      

      dx = dX * patSize;
      dy = dY * patSize;

      //bounce point here
      if (boundaryNextPoint (pp -> x, pp -> y, dx, dy, dim, io) == 1)
	{
	  possibleX = pp -> x + dx;
	  possibleY = pp -> y + dy;

	  //if it bounced, get & set the new angle
	  if ((double (possibleX) - double (dim.w/2) > 0.01) || 
	      (double (possibleY) - double (dim.h/2) > 0.01))
	    {
	      rectToPolar (double (dim.w/2) - double (possibleX), double (dim.h/2) - double (possibleY), rad2, theta);

	      //add some randomization there...
	      dir = theta + ((randomUniform () * numPI_4) - (numPI_4 / 2.0));	      
	    }
	  else
	    {dir = randomUniform () * 2.0 * numPI; }
	}

      pp -> x += Sint32 (dx);
      pp -> y += Sint32 (dy);

      //calculate the total length.
      if (i > 0)
	{
	  distanceBetweenDots.at (i - 1) = hypotCalc (double (pp->x - pp2.x), double (pp->y - pp2.y));
	  //determine line density here
	  pp -> lengthOfLine = 64;
	}
      else
	pp -> lengthOfLine = 64;

      pp2.x = pp -> x;
      pp2.y = pp -> y;

      if (!(i == 0 && ignoreFirst))
	newP.push_back (pp);
    }

  //now for the important points: Evenly spread out across the length of the pattern.
  lengthOfPattern = 0.0;
  for (i = 0; i < distanceBetweenDots.size (); i ++)
    lengthOfPattern += distanceBetweenDots.at (i);

  for (i = 0; i < distanceBetweenDots.size (); i ++)
    {
      relativeLength = (distanceBetweenDots.at(i)/lengthOfPattern * 255);
      if (relativeLength < 1.0)
	newP.at (firstIndex + i) -> lengthOfLine = 0;
      else
	newP.at (firstIndex + i) -> lengthOfLine = Uint32 (relativeLength) - 1;
    }

  //change dir to the last one made.
  dir = theta1;

  return lasti;
}

Uint32 LinePattern :: boundaryNextPoint (Sint32 & newPtx, Sint32 & newPty, double & dx, double & dy, SDL_Rect & dim, InputObj * io)
{
  //checks to see if the point newPt + d is out of bounds (dim)
  //if so, changes d accordingly, and the function returns 1.

  Uint32 bounced = 0;

  dir = restrictAnglePos (dir);
  
  if (newPtx + Sint32(dx) < 0.0 || newPtx + Sint32(dx) > dim.w)
    { dx *= -1.0; bounced = 1;}

  if (newPty + Sint32(dy) < 0.0 || newPty + Sint32(dy) > dim.h)
    { dy *= -1.0; bounced = 1;}

  //clipping
  if (newPtx < 0)
    dx = -1.0 * newPtx;
  if (newPtx > dim.w)
    dx = dim.w - newPtx;
  if (newPty < 0)
    dy = -1.0 * newPty;
  if (newPty > dim.h)
    dy = dim.h - newPty;
  
  //clip dir
  dir = restrictAnglePos (dir);
  return bounced;
}

bool LinePattern :: mouseOverPoint (InputObj * io, patternPoint * point, SDL_Rect & dim)
{
  //checks to see if the mouse is:
  //1) clicked.
  //2) that point -> type is not "used" (2.)
  //3) hovering over the given point.
  //4) Interpolating between the last known mouse position and the current mouse position. This is to help solve the "lag" issues inbetween animation frames.

  Sint32 dx, dy;
  Sint32 distSq, totalAcc;

  Sint32 mousePosX = io -> getCursorPosX (), mousePosY = io -> getCursorPosY ();
  Sint32 mousePrevPosX = io -> getPreviousCursorPosX (), mousePrevPosY = io -> getPreviousCursorPosY ();

  Sint32 ppointX = point -> x + dim.x, ppointY = point -> y + dim.y;
  Sint32 distToLine, xhatTop, xhatBottom;

  if (!io -> getButton1 ())
    return false;

  if (point -> type == 2 || point -> type == 3)
    return false;

  //is it at the current point?
  dx = point -> x + dim.x - mousePosX;
  dy = point -> y + dim.y - mousePosY;

  distSq = (dx * dx) + (dy * dy);
  totalAcc = io -> getAccuracy () + point -> size;

  //if mouse is at (or close enough...)
  if (distSq <= (totalAcc * totalAcc))
      return true;

  //To make up for lag, we need to check to see if this point is between the last two mouse positions.
  //There are two criterion that must be true if the interpolation works out.
  //1) the distance from the point to the line the mouse positions form is close enough
  //2) whether the point, projected to a line, is between the mouse points.

  //First, if the two mouse points are the same, there's not much we can do.
  if (mousePosX == mousePrevPosX && mousePosY == mousePrevPosY)
    return false;

  //Now we must normalize. Shift mousePos to the origin.
  mousePosX -= mousePrevPosX;
  mousePosY -= mousePrevPosY;

  ppointX -= mousePrevPosX;
  ppointY -= mousePrevPosY;

  //wheee, line projection distance calculation
  distToLine = (ppointX * mousePosY) - (mousePosX * ppointY);
  distToLine *= distToLine;
  distToLine /= (mousePosX * mousePosX) + (mousePosY * mousePosY);

  //distToLine is the square of the distance to the line. If it's more than the mouse's accuracy, it's too far.

  if (distToLine > (totalAcc * totalAcc))
    return false;

  //OK. Now we need to see if point's within the line segment between the mouse points. We're going to use a projection (linear algebra style) to get xhat.

  //It's already been normalized, so...

  xhatTop = (mousePosX * ppointX) + (mousePosY * ppointY);
  xhatBottom = (mousePosX * mousePosX) + (mousePosY * mousePosY);

  //xhat = xhatTop/xhatBottom. If xhat < 0 or > 1, then no. Otherwise, yes!
  if (xhatTop < 0 || xhatTop > xhatBottom)
    return false;
  else
    return true;

  return false;
}

LinePattern :: ~LinePattern ()
{
  vector <patternPoint *> :: iterator iter;

  iter = easyPoints.begin ();
  while (iter < easyPoints.end ())
    {delete (*iter); iter++;}
  easyPoints.clear ();

  iter = hardPoints.begin ();
  while (iter < hardPoints.end ())
    {delete (*iter); iter++;}
  hardPoints.clear ();
}
