#ifndef LINEPATTERN_H
#define LINEPATTERN_H

#include "SDL.h"
#include "SDL_gfxPrimitives.h"
#include "InputObj.h"
#include <vector>
using namespace std;

struct patternPoint {
  patternPoint ()
  {  }

  Sint32 x;
  Sint32 y;
  Uint8 size;

  Uint8 type;
  // 0 = big, 1 = small, 2 = used, 3 = invisible
  Uint8 bonus;
  // 0 = none, 1 = attack, 2 = defense, 3 = heal
  Uint8 side;
  // 0 = none, 1 = easy, 2 = hard
  Uint8 lengthOfLine;
};

class LinePattern {
 public:
  LinePattern ();
  ~LinePattern ();

  //rendering
  void render (SDL_Surface * dest, SDL_Rect & dim, Uint8, Uint8);
  void renderPoint (SDL_Surface *, SDL_Rect &, patternPoint *, Uint8, Uint8);
  void renderVector (SDL_Surface *, SDL_Rect &, vector <patternPoint *> *, Uint8);
  void renderLine (SDL_Surface * dest, SDL_Rect & dim, patternPoint * pp, patternPoint * pp2, Uint8 intensity, Uint8);

  //hit detection
  Uint8 checkOnFirstPoint (InputObj *, SDL_Rect &);
  Uint8 checkOnPoints (InputObj *, SDL_Rect &, Uint32 &, Uint32 &);
  bool checkVec (InputObj *, SDL_Rect &, vector <patternPoint *> *, Uint32 &, Uint32 &);
  bool mouseOverPoint (InputObj *, patternPoint *, SDL_Rect &);

  //generating patterns
  void setInitialPatterns (SDL_Rect &);
  Uint8 generatePatterns (InputObj *, SDL_Rect &, Uint8 &, double, Uint32, double , Uint32, Sint32);

 protected:
  void loadShape (vector <Sint32> &, vector <Sint32> &, Uint32, patternPoint *, Uint32 &, Uint32 &);
  void getSplinePoints (vector <Sint32> &, vector <Sint32> &, Uint32);
  Uint32 addPatternPoints (vector <patternPoint *> &, vector <Sint32> &, vector <Sint32> &, patternPoint *, SDL_Rect &, Uint8, Uint8, double, InputObj *, bool);
  Uint32 boundaryNextPoint (Sint32 &, Sint32 &, double &, double &, SDL_Rect &, InputObj *);

  Uint8 activePoints; //0 = just the origin, 1 = easy and hard, 2 = only easy, 3 = only hard
  patternPoint originPoint;
  vector <patternPoint *> easyPoints;
  vector <patternPoint *> hardPoints;
  vector <patternPoint *> newPoints;

  SDL_Event lpEvent;
  Uint32 statBoosted;
  Uint32 statBoostedAmnt;
  double dir;
};
#endif
