#include "MediaPlayer.h"

#include "SDL.h"
#include "SDL_image.h"

#include <vector>
#include <iostream>
#include <string>
using namespace std;

#include "MediaResource.h"

MediaPlayer :: MediaPlayer (MediaResource * mr)
{
  medRes = mr;
  animCatToPlay = 0;
  animIndexToPlay = 0;
  channelToPlay = 0;
  startTime = SDL_GetTicks ();
  loopCounter = 0;
  animSubToPlay = 0;
  mpPartner = NULL;
  partnerCategoryAnim = 0;
  partnerSubAnim = 0;
  partnerAnimated = false;
}

int MediaPlayer :: render (SDL_Surface * dest, SDL_Rect & center)
{
  //calls medRes for the next picture and pastes it.
  //also increments the frame if it needs to.

  medRes -> drawPic (dest, center, animCatToPlay, animSubToPlay, animIndexToPlay);

  //tell the mp partner to start animating.
  if (mpPartner != NULL && (partnerAnimated == false) && medRes -> getPartnerShouldAnimateNow (animCatToPlay, animSubToPlay, animIndexToPlay))
    { mpPartner -> changeAnim (partnerCategoryAnim, partnerSubAnim);
      partnerAnimated = true;}

  //check if it's time to change frames.
  Sint32 changePicStatus = medRes -> changePic (SDL_GetTicks () - startTime, animCatToPlay, animSubToPlay, animIndexToPlay, loopCounter);

  if (changePicStatus != 0)
    {
      startTime = SDL_GetTicks (); partnerAnimated = false;
      if (changePicStatus == 4)
	mpPartner = NULL;
    }
  return 0;
}

int MediaPlayer :: changeAnim (Uint32 index, Uint32 subIndex, MediaPlayer * partnerMP, Uint32 partnerAnim, Uint32 partnerSAnim)
{
  animCatToPlay = index;
  animSubToPlay = subIndex;
  animIndexToPlay = 0;
  startTime = SDL_GetTicks ();
  loopCounter = medRes -> getLoops (index, subIndex);
  
  //set up a partnered media player
  mpPartner = partnerMP;
  partnerCategoryAnim = partnerAnim;
  partnerSubAnim = partnerSAnim;

  partnerAnimated = false;
  return 0;
}

Uint32 MediaPlayer :: numberOfPossibleAnims (Uint32 anim)
{
  return medRes -> numberOfPossibleAnims (anim);
}

MediaPlayer :: ~MediaPlayer ()
{
}
