#include "MediaResource.h"

#include "SDL.h"
#include "SDL_image.h"
//#include "SDL_mixer.h"

#include <vector>
#include <string>
#include <iostream>
#include <fstream>
#include <ostream>
#include <sstream>
using namespace std;

#include "AnimFrame.h"

#define animIdleIndex 0
#define animAttackIndex 1
#define animPainIndex 2
#define animDeadIndex 3
#define animIntroIndex 4
#define animGloatIndex 5
#define animIndexMax 6

MediaResource :: MediaResource (string n)
{
  //loads stuff.
  //looks for the file: media/chars/n/info (n is what you pass in.)
  //each line in info is of the form:
  //Type Media MediaSpecificStuff
  //examples:
  //Idle 0 -1 0 <-- Media 0: Main info about the animation
  //Attk 1 10 20 900 0 <--- Media 1: Animation frame.
  //Dead 2 scream.ogg <---- Media 2: Sounds.

  string bufString;
  string activity;
  Uint32 mediaType;
  Sint32 intReader;
  Uint32 uintReader;
  Uint8 animType = 255;

  Uint32 picIndex = 0;
  string tempStr;

  Sint32 xo; Sint32 yo; Uint32 to; bool pp;

  ostringstream fileName (ostringstream::out);
  fileName << "media/chars/" << n << "/info";

  actionList.resize (animIndexMax);

  vector <vector <action *> > :: iterator iter;
  iter = actionList.begin ();

  while (iter < actionList.end ())
    {
      (*iter).resize (0);
      iter ++;
    }

  action * tempAction = NULL;
  AnimFrame * tempAF = NULL;

  //load the file.
  infile.open (fileName.str().c_str());
  if (!infile.is_open ())
    {cout << "Cannot find file:" << fileName.str () << endl; 
    cout.flush (); return;}

  name.assign (n);
  picIndex = 0;

  while (!infile.eof ())
    {
      infile >> bufString;
      activity = bufString;
      infile >> mediaType;

      //call subfuctions depending on mediaType.

      tempAction = NULL; tempAF = NULL;

      if (bufString.compare ("Idle") == 0)
	animType = animIdleIndex;
	  
      if (bufString.compare ("Attk") == 0)
	animType = animAttackIndex;

      if (bufString.compare ("Pain") == 0)
	animType = animPainIndex;

      if (bufString.compare ("Dead") == 0)
	animType = animDeadIndex;
	  
      if (bufString.compare ("Gloat") == 0)
	animType = animGloatIndex;

      if (bufString.compare ("Intro") == 0)
	animType = animIntroIndex;

      if (animType < animIndexMax)
	{
	  switch (mediaType){
	  case (0):
	    //make a new action type. 0
	    //Format: Number of loops (-1 is infinite), What to do when out of loops
	    tempAction = new action ();
	    infile >> intReader;
	    tempAction -> numLoops = intReader;
	    infile >> uintReader;
	    tempAction -> whatToDoWhenOutOfLoops = uintReader;
	    actionList.at(animType).push_back (tempAction);
	    picIndex = 0;
	    break;
	  case (1):
	    //load animations
	    //construct the name, look for it.
	    tempAF = NULL;
	    tempStr = "";
	    fileName.str (tempStr);
	    fileName << "media/chars/" << n << "/";
		
	    switch (animType)
	      {
	      case (animIdleIndex): fileName << "Idle"; break;
	      case (animAttackIndex): fileName << "Attk"; break;
	      case (animPainIndex): fileName << "Pain"; break;
	      case (animDeadIndex): fileName << "Dead"; break;
	      case (animIntroIndex): fileName << "Intro"; break;
	      case (animGloatIndex): fileName << "Gloat"; break;
	      default: break;
	      }
	    
	    fileName << (actionList.at (animType).size () - 1) << "-" << picIndex <<".png";
		
	    //the AnimFrame does the actual loading
	    tempAF = new AnimFrame (fileName.str ());
	    if (tempAF -> isValid ())
	      {
		infile >> xo; infile >> yo; infile >> to;
		infile >> intReader;
		pp = (intReader == 0);
		tempAF -> setOffsetAndTime (xo, yo, to, pp);
		actionList.at (animType).back () -> anim.push_back (tempAF);
		picIndex ++;
	      }
	    else
	      { 
		cerr << "Did not load " << fileName.str () << endl;
		delete tempAF;
	      }
	    break;
	  case (2):
	    //import sounds
	    break;
	  default: break;
	  };
	}
    }    
}

Sint32 MediaResource :: changePic (Uint32 timeElapsed, Uint32 & animIndex, Uint32 & subIndex, Uint32 & frameIndex, Sint32 & numLoops)
{
  //changes the pic.
  //returns 0 if it didn't change the pic.
  //returns 1 if it incremented the pic.
  //returns 2 if it reached the end of the pics and was looping over.
  //returns 3 if it reached the end of the pics and was holding the last frame.
  //returns 4 if it reached the end and was resetting to the idle frame.

  Sint32 ans = 0;

  Uint32 timeToHoldFrame = getAction (animIndex, subIndex) -> anim.at (frameIndex) -> getTime ();

  if (timeElapsed >= timeToHoldFrame)
    {
      //increment frame.
      frameIndex ++;
      ans = 1;

      //check if it's too far
      if (frameIndex >= getAction (animIndex, subIndex) -> anim.size ())
	{
	  //decrease the number of loops...maybe

	  if (numLoops <= -1)
	    {
	      //infinite loops.
	      frameIndex = 0;
	      ans = 2;
	    }
	  else if (numLoops == 0)
	    {
	      //decide what to do here.
	      switch (getAction (animIndex, 0) -> whatToDoWhenOutOfLoops){
	      case 0: //just cancel the animation
		animIndex = 0;
		subIndex = 0;
		frameIndex = 0;
		ans = 4;
		break;
	      case 1: //hold it on the last frame
		frameIndex --;
		ans = 3;
		break;
	      default:
		animIndex = 0;
		subIndex = 0;
		frameIndex = 0;
		ans = 2;
		break;
	      };
	    }
	  else
	    {
	      //just loop and try again
	      numLoops --;
	      frameIndex = 0;
	      ans = 2;
	    }
	}
    }
  return ans;
}

Sint32 MediaResource :: drawPic (SDL_Surface * dest, SDL_Rect & center, Uint32 animIndex, Uint32 subIndex, Uint32 frameIndex)
{
  //gets the next frame.
  getAction (animIndex, subIndex) -> anim.at (frameIndex) -> render (dest, center);
  return 0;
}

Sint32 MediaResource :: getLoops (Uint32 animIndex, Uint32 subIndex)
{
  return getAction (animIndex, subIndex) -> numLoops;
}

bool MediaResource :: getPartnerShouldAnimateNow (Uint32 animCatIndex, Uint32 subAnim, Uint32 frame)
{
  bool ans = getAction (animCatIndex, subAnim) -> anim.at (frame) -> getPlayPartner ();
  return ans;
}

Uint32 MediaResource :: numberOfPossibleAnims (Uint32 animCat)
{
  return actionList.at (animCat).size ();
}

action * MediaResource :: getAction (Uint32 animCat, Uint32 subIndex)
{
  if (animCat >= actionList.size ())
    return actionList.at (0).front ();

  if (subIndex >= actionList.at (animCat).size ())
    {
      if (actionList.at (animCat).size () == 0)
	return actionList.at (0).front ();

      return actionList.at (animCat).at (0);
    }

  return actionList.at (animCat).at (subIndex);
}

MediaResource :: ~MediaResource ()
{
  infile.close ();
  //and the pics?
  vector <vector <action *> > :: iterator iter;
  vector <action *> :: iterator iter2;

  iter = actionList.begin ();

  while (iter < actionList.end())
    {
      iter2 = (*iter).begin ();
      while (iter2 < (*iter).end ())
	{
	  delete (*iter2);
	  iter2++;
	}
      iter++;
    }
}
