#include "SDL.h"
#include "SDL_ttf.h"

#include <stdlib.h>
#include <time.h>

#include <iostream>
#include <vector>
#include <string>
using namespace std;

#include "InputObj.h"
#include "UserInput.h"
#include "Battle.h"
#include "Combatant.h"
#include "MediaResource.h"
#include "MediaPlayer.h"

SDL_Surface * screen;
TTF_Font * font;
int screenResX = 640, screenResY = 480;

SDL_Event mainEvent;

Battle * battul;
UserInput masterInput;
Combatant * lComb = NULL;
Combatant * rComb = NULL;

bool escKeyHit = false;
bool resetGameHit = false;
int gameMode = 0;

const char title [] = "SwordDrawBattle";
const char icon [] = "SwordDrawBattle";

//temp!
MediaResource * mr;
MediaPlayer * mo;

bool fullscreenRender = true;
//#include <mcheck.h>

#define videoflags SDL_SWSURFACE
//#define videoflags SDL_HWSURFACE | SDL_DOUBLEBUF

void resetGame ()
{
  string playerName ("Player");

  //setup screen
  if (lComb != NULL)
    delete lComb;

  lComb = new Combatant ();
  lComb -> setInput (&masterInput);
  lComb -> setName (playerName);
  lComb -> setMaxHP (100);
  lComb -> setMedia (mo);

  battul -> reset ();

  gameMode = 1; //set to battle mode
  resetGameHit = false;
}

Sint8 render ()
{
  //draw stuff on screen

  //once it has delayed, draw to the screen and flip it.

  //call Battle to render this...if in BattleMode.
  if (gameMode == 0)
    SDL_FillRect(screen, NULL, SDL_MapRGB(screen -> format, 0, 0, 0));
  else if (gameMode == 1)
    battul -> render (screen);

  //finally, flip the screen?
  if (SDL_Flip(screen) == -1)
    {
      printf("Cannot flip screens: %s\n", SDL_GetError ());
      return -1;
    }
  return 0;
}

Sint8 gameLoop()
{
  bool done = false;

  while (!done)
    {
      //get events until the queue is empty
      while (SDL_PollEvent(&mainEvent) == 1)
	{
	  //for each event:
	  switch(mainEvent.type){
	  case SDL_KEYDOWN:
	    //check to see if it wants to quit	    
	    if(mainEvent.key.keysym.sym==SDLK_ESCAPE)
	      escKeyHit = true;
	    break;
	  case SDL_KEYUP:
	    //check to see if it wants to quit	    
	    if(mainEvent.key.keysym.sym==SDLK_ESCAPE)
	      escKeyHit = false;
	    if(mainEvent.key.keysym.sym==SDLK_F10)
	      resetGameHit = true;
	    if(mainEvent.key.keysym.sym==SDLK_f)
	      {
		SDL_FreeSurface(screen);
		if (fullscreenRender)
		  {
		  screen = SDL_SetVideoMode(screenResX, screenResY, 32, videoflags);
		  fullscreenRender = false;
		  }
		else
		  {
		  screen = SDL_SetVideoMode(screenResX, screenResY, 32, videoflags|SDL_FULLSCREEN);
		  fullscreenRender = true;
		  }
	      }
	    break;
	  case SDL_APPMOUSEFOCUS:
	    if (mainEvent.active.gain == 1)
	      masterInput.forceMousePos ();
	    break;
	  default:
	    break;
	  }

	  if (gameMode == 0)
	    masterInput.processInput (mainEvent);
	  else if (gameMode == 1)
	    battul -> processInput (mainEvent);
	}
      //done with all events

      //tell battle to do stuff here
      battul -> gameLoop ();

      //check to see if it wants to quit
      if(escKeyHit)
	done = true;
      
      //make sure this is only when in Battul Mode.

      //put reset key here
      if (resetGameHit)
	resetGame ();

      if (gameMode == 1 && battul -> getStatus () == 0)
	battul -> init (lComb, font);

      //perform results of logic here

      //draw the screen
      if (render () == -1)
	return -1;
    }
  return 0;
}

Sint8 loadRound()
{
  //set up the rules.
  return 0;
}

Sint8 initGameVariables()
{
  //init keyboard
  SDL_EnableKeyRepeat(1, 30);

  //set the randomizer seed.
  srand( time(NULL) );
  return 0;
}

Sint8 initGame()
{
  if (initGameVariables() == -1)
    return -1;

  if (loadRound () == -1)
    return -1;

  return 0;
}

Sint8 initGfx(bool debugMode)
{
  //set up screen

  if (debugMode)
    screen = SDL_SetVideoMode(screenResX, screenResY, 32, videoflags|SDL_FULLSCREEN);    
  else
    screen = SDL_SetVideoMode(screenResX, screenResY, 32, videoflags);

  if (screen == NULL)
    {
      printf ("Error initializing video: %s", SDL_GetError ());
      return -1;
    }

  //load graphics here.
  //temp and stuff.

  string fn1 ("demoChar");
  
  mr = new MediaResource (fn1);
  mo = new MediaPlayer (mr);

  //reset game
  resetGame ();

  //disbale mouse
  SDL_ShowCursor(SDL_DISABLE);
  return 0;
}

Sint8 initSDL(bool debugMode)
{
  if (SDL_Init(SDL_INIT_VIDEO) == Sint8(-1))
    {
      printf ("Unable to init SDL: %s\n", SDL_GetError());
      return -1;
    }

  if(TTF_Init()==-1) 
    {
      printf("Unable to init SDL_TTF: %s\n", TTF_GetError());
      return -1;
    }

  font=TTF_OpenFont("text/FreeSans.ttf", 14);
  
  if(font == NULL) 
    {
      printf("TTF_OpenFont: %s\n", TTF_GetError());
      return -1;
    }
  return 0;
}

Sint8 initEngine()
{
  if (initSDL(fullscreenRender) == -1)
    return -1;

  if (initGfx (fullscreenRender) == -1)
    return -1;

  //window futzing
  SDL_WM_SetCaption(title, icon);

  return 0;
}

int main (int argc, char *argv[])
{
  /*
  if (fullscreenRender)
    mtrace ();
  */

  battul = new Battle ();

  if (initEngine() == -1)
    return -1;

  if (initGame () == -1)
    return -1;

  if (gameLoop () == -1)
    return -1;

  SDL_ShowCursor(SDL_ENABLE);

  delete lComb;
  delete rComb;

  //temp 
  delete mr;
  delete mo;

  delete battul;
  TTF_CloseFont(font);
  TTF_Quit ();
  SDL_FreeSurface(screen);
  SDL_Quit ();
  return 0;
};
